﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Threading;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class CommonResourceExample
	{
		static readonly Mutex mutex = new Mutex(false, "CommonResourceExample");
		static readonly string fileName = "CommonResourceExample.txt";

		public string ReplaceFileContents(string fileContent)
		{
			try
			{
				mutex.WaitOne();
				string result = ReadFileCore();
				WriteFileCore(fileContent);
				return result;
			}
			finally
			{
				mutex.ReleaseMutex();
			}
		}

		public void WriteFile(string fileContent)
		{
			try
			{
				mutex.WaitOne();
				WriteFileCore(fileContent);
			}
			finally
			{
				mutex.ReleaseMutex();
			}
		}

		void WriteFileCore(string fileContent)
		{
			using (IsolatedStorageFile isolatedStorageFile
							= IsolatedStorageFile.GetUserStoreForApplication())
			{
				using (IsolatedStorageFileStream outStream
							= isolatedStorageFile.OpenFile(fileName, FileMode.Create))
				{
					using (StreamWriter writer = new StreamWriter(outStream))
					{
						writer.Write(fileContent);
					}
				}
			}
		}

		public string ReadFile()
		{
			try
			{
				mutex.WaitOne();
				return ReadFileCore();
			}
			finally
			{
				mutex.ReleaseMutex();
			}
		}

		string ReadFileCore()
		{
			using (IsolatedStorageFile isolatedStorageFile
						= IsolatedStorageFile.GetUserStoreForApplication())
			{
				if (!isolatedStorageFile.FileExists(fileName))
				{
					return string.Empty;
				}

				using (IsolatedStorageFileStream inStream
							= isolatedStorageFile.OpenFile(fileName, FileMode.Open))
				{
					using (StreamReader reader = new StreamReader(inStream))
					{
						return reader.ReadToEnd();
					}
				}
			}
		}

		//Random random = new Random();

		public string WriteAndReadBackFile(string fileContent)
		{
			try
			{
				mutex.WaitOne();
				WriteFileCore(fileContent);
				//int sleepTime = random.Next(1, 500);
				//Thread.Sleep(sleepTime);
				string result = ReadFileCore();
				return result;
			}
			finally
			{
				mutex.ReleaseMutex();
			}
		}
	}
}
