﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;

using Microsoft.Phone.Scheduler;

namespace DanielVaughan.WindowsPhone7Unleashed.ReminderSetter
{
	public class ReminderViewModel : ViewModelBase
	{
		public const string IdQueryKey = "Id";

		public ReminderViewModel()
		{
			reminderSetCommand = new DelegateCommand(obj => SetReminder());
			loadReminderCommand = new DelegateCommand<string>(LoadReminder);
		}

		void LoadReminder(string name)
		{
			Reminder reminder = ScheduledActionService.Find(name) as Reminder;
			if (reminder == null)
			{
				MessageService.ShowError("Reminder not found.");
				return;
			}

			ReminderBeginTime = reminder.BeginTime;
			ReminderContent = reminder.Content;
			ReminderName = reminder.Name;
			ReminderRecurrenceType = reminder.RecurrenceType;
			ReminderTitle = reminder.Title;
		}

		void SetReminder()
		{
			const string reminderUrlFormat = "/Reminders/ReminderView.xaml?Id={0}";
			string navigationUrl = string.Format(reminderUrlFormat, ReminderName/*Guid.NewGuid().ToString("N")*/);
			Uri navigationUri = new Uri(navigationUrl, UriKind.Relative);

			Reminder reminder = new Reminder(ReminderName)
			              	{
			              		BeginTime = reminderBeginTime, 
								ExpirationTime = reminderExpirationTime,
			              		Content = reminderContent, 
								RecurrenceType = reminderRecurrenceType,
								NavigationUri = navigationUri,
								Title = reminderTitle
			              	};
//			Reminder reminder = new Reminder("ReminderName")
//			{
//				BeginTime = DateTime.Now.AddMinutes(5),
//				ExpirationTime = DateTime.Now.AddDays(5),
//				Content = "Your daily reminder.",
//				RecurrenceType = RecurrenceInterval.Daily,
//				NavigationUri = new Uri("/ReminderView.xaml?Id=ReminderName"),
//				Title = reminderTitle
//			};

			if (ScheduledActionService.Find(reminder.Name) != null)
			{
				ScheduledActionService.Remove(reminder.Name);
			}

			ScheduledActionService.Add(reminder);

			MessageService.ShowMessage("reminder set");

//			for (int i = 0; i < 140; i++)
//			{
//				Reminder reminder2 = new Reminder("ReminderName" + i)
//				{
//					BeginTime = DateTime.Now.AddSeconds(60 + i),
//					ExpirationTime = DateTime.Now.AddDays(5),
//					Content = "Your daily reminder " + i,
//					RecurrenceType = RecurrenceInterval.Daily,
//					NavigationUri = new Uri("/ReminderView.xaml?Id=ReminderName" + i , UriKind.Relative),
//					Title = reminderTitle
//				};
//				ScheduledActionService.Add(reminder2);
//			}
		}

		readonly DelegateCommand reminderSetCommand;

		public ICommand ReminderSetCommand
		{
			get
			{
				return reminderSetCommand;
			}
		}

		DateTime reminderBeginTime = DateTime.Now.AddSeconds(30);

		public DateTime ReminderBeginTime
		{
			get
			{
				return reminderBeginTime;
			}
			set
			{
				Assign(() => ReminderBeginTime, ref reminderBeginTime, value);
			}
		}

		DateTime reminderExpirationTime = DateTime.Now.AddDays(14);

		public DateTime ReminderExpirationTime
		{
			get
			{
				return reminderExpirationTime;
			}
			set
			{
				Assign(() => ReminderExpirationTime, ref reminderExpirationTime, value);
			}
		}

		string reminderName = "default reminder";

		public string ReminderName
		{
			get
			{
				return reminderName;
			}
			set
			{
				Assign(() => ReminderName, ref reminderName, value);
			}
		}

		string reminderTitle = "default reminder";

		public string ReminderTitle
		{
			get
			{
				return reminderTitle;
			}
			set
			{
				Assign(() => ReminderTitle, ref reminderTitle, value);
			}
		}

		readonly Uri reminderSound = new Uri("/Sounds/Alarm.wma", UriKind.Relative);

		public Uri ReminderSound
		{
			get
			{
				return reminderSound;
			}
		}

		string reminderContent = "test content";

		public string ReminderContent
		{
			get
			{
				return reminderContent;
			}
			set
			{
				Assign(() => ReminderContent, ref reminderContent, value);
			}
		}

		RecurrenceInterval reminderRecurrenceType;

		public RecurrenceInterval ReminderRecurrenceType
		{
			get
			{
				return reminderRecurrenceType;
			}
			set
			{
				Assign(() => ReminderRecurrenceType, ref reminderRecurrenceType, value);
			}
		}

		readonly IEnumerable<RecurrenceInterval> recurrenceIntervals 
			= CreateEnumValueList<RecurrenceInterval>();

		public IEnumerable<RecurrenceInterval> RecurrenceIntervals
		{
			get
			{
				return recurrenceIntervals;
			}
		}

		static IEnumerable<TEnum> CreateEnumValueList<TEnum>()
		{
			Type enumType = typeof(TEnum);
			var fieldInfos = enumType.GetFields().Where(x => enumType.Equals(x.FieldType));
			return fieldInfos.Select(
				fieldInfo => (TEnum)Enum.Parse(enumType, fieldInfo.Name, true)).ToList();
		}

		readonly DelegateCommand<string> loadReminderCommand;

		public DelegateCommand<string> LoadReminderCommand
		{
			get
			{
				return loadReminderCommand;
			}
		}
	}
}
