﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading;
using System.Windows;

using DanielVaughan.Network;

using Microsoft.Phone.Scheduler;
using Microsoft.Phone.Shell;

using DanielVaughan.WindowsPhone7Unleashed.TodoList;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class TaskScheduler : ScheduledTaskAgent
	{
		internal const string TodoItemIdQueryKey = "TodoItemId";
		readonly Mutex mutex = new Mutex(false, "todo");

		/// <summary>
		/// Agent that runs a scheduled task
		/// </summary>
		/// <param name="task">
		/// The invoked task
		/// </param>
		/// <remarks>
		/// This method is called when a periodic or resource intensive task is invoked
		/// </remarks>
		protected override void OnInvoke(ScheduledTask task)
		{
//			ShellToast toast = new ShellToast
//			                   	{
//			                   		Title = "Windows Phone 7 Unleashed",
//									Content = "Task Scheduler running..."
//			                   	};
//			toast.Show();

			/* Detect if the task is periodic or resource intensive. */
			if (task is PeriodicTask)
			{
				ProcessPeriodicTask();
			}
			else
			{
				ProcessResourceIntensiveTask();
			}

			/* Perform operation on common resource such as isolated storage. */
			bool mutexAcquired = false;
			try
			{
				mutexAcquired = mutex.WaitOne(10000);
				if (mutexAcquired)
				{
					/* Access common resource here. */
					Debug.WriteLine("mutex entered.");
				}
			}
			finally
			{
				if (mutexAcquired)
				{
					mutex.ReleaseMutex();
				}
			}

			if (!mutexAcquired)
			{
				/* Unable to acquire ownership of mutex. */
				Debug.WriteLine("mutex timed out.");
			}

			NotifyComplete();
		}

		void ProcessPeriodicTask()
		{
			TodoService todoService = new TodoService();

			string queryWithEquals = TodoItemIdQueryKey + "=";
			IEnumerable<ShellTile> tiles = ShellTile.ActiveTiles.Where(
					tile => tile.NavigationUri.ToString().Contains(queryWithEquals));

			foreach (var tile in tiles)
			{
				/* NavigationUri.Query raises an exception on relative URIs. */
				Dictionary<string, string> queryPairs
					= UrlUtility.ParseQueryString(tile.NavigationUri.ToString()); 

				string idString = queryPairs[TodoItemIdQueryKey];
				int todoItemId;
				if (!int.TryParse(idString, out todoItemId))
				{
					Debug.Assert(false, "Invalid id " + idString);
					continue;
				}

				TodoItem todoItem;
				if (!todoService.TryGetTodoItem(todoItemId, out todoItem))
				{
					Debug.Assert(false, "Unknown item " + idString);
					continue;
				}

				StandardTileData tileData = TodoTileDataCreator.CreateTile(
																	todoItem.Description,
																	todoItem.DueDate);
				tile.Update(tileData);
			}

			//			int byteSize = 100000000;
			//			var bytes = new byte[byteSize];
			//			for (int i = 0; i < byteSize; i++)
			//			{
			//				bytes[i] = new byte();
			//			}

			long usedBytes = Microsoft.Phone.Info.DeviceStatus.ApplicationCurrentMemoryUsage;
			long limitBytes = Microsoft.Phone.Info.DeviceStatus.ApplicationMemoryUsageLimit;
			if ((usedBytes + 1048576) < limitBytes) // 1 048 576 = 1 MB
			{
				/* Do work with 1 MB. */
			}
			string message = string.Format("{0} of {1} bytes used.", usedBytes, limitBytes);
			Console.WriteLine(message);

			ShellToast toast = new ShellToast
			{
				Title = "Memory",
				Content = message
			};
			toast.Show();

            //            long totalMemoryBytes = Microsoft.Phone.Info.DeviceStatus.DeviceTotalMemory;
            //            long peakUsageBytes = Microsoft.Phone.Info.DeviceStatus.ApplicationPeakMemoryUsage;
            //            if (totalMemoryBytes < 90000000)
            //            {
            //                /* Restrain memory usage. */
            //            }
		}

		void ProcessResourceIntensiveTask()
		{
		}
	}
}
