﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace DanielVaughan.WindowsPhone7Unleashed.TodoList
{
	public class TodoService : ITodoService
	{
		readonly TodoDatabaseUtility databaseUtility = new TodoDatabaseUtility();
		TodoDataContext dataContext;

		readonly List<TodoItem> dummyList = new List<TodoItem> {
			new TodoItem { Description = "check-in project", DueDate = DateTime.Now.AddDays(1)},
			new TodoItem { Description = "call client", DueDate = DateTime.Now.AddDays(1)},
			new TodoItem { Description = "buy milk", DueDate = DateTime.Now.AddDays(2)},
			new TodoItem { Description = "order book", DueDate = DateTime.Now.AddDays(-2)}};

		public TodoService()
		{
			Initialize();
		}

		public void Initialize()
		{
			if (dataContext != null)
			{
				dataContext.Dispose();
			}
			databaseUtility.InitializeDatabase();
			dataContext = databaseUtility.CreateContext();

			/* For demonstration purposes. */
			if (GetTodoItems().Count() < 1)
			{
				dataContext.TodoItems.InsertAllOnSubmit(dummyList);
				dataContext.SubmitChanges();
			}
		}

		public IEnumerable<TodoItem> GetTodoItems()
		{
			return dataContext.TodoItems;
		}

		public TodoItem GetTodoItem(int itemId)
		{
			TodoItem item = dataContext.TodoItems.Where(
							todoItem => todoItem.Id == itemId).FirstOrDefault();

			if (item == null)
			{
				throw new KeyNotFoundException(
					string.Format("Item with key '{0}' was not found.", itemId));
			}

			return item;
		}

		public bool TryGetTodoItem(int itemId, out TodoItem todoItem)
		{
			try
			{
				todoItem = GetTodoItem(itemId);
				return todoItem != null;
			}
			catch (Exception)
			{
				todoItem = null;
			}
			return false;
		}

		public void AddOrUpdateItem(TodoItem todoItem)
		{
			ArgumentValidator.AssertNotNull(todoItem, "todoItem");

			if (todoItem.Id <= 0)
			{
				dataContext.TodoItems.InsertOnSubmit(todoItem);
			}
			dataContext.SubmitChanges();
		}

		public void RemoveTodoItem(TodoItem todoItem)
		{
			ArgumentValidator.AssertNotNull(todoItem, "todoItem");

			dataContext.TodoItems.DeleteOnSubmit(todoItem);
			dataContext.SubmitChanges();
		}
	}
}
