using System;
using System.IO;

using DanielVaughan.Data;

using Media;

using Microsoft.Phone.BackgroundAudio;

namespace DanielVaughan.WindowsPhone7Unleashed.BackgroundAudio
{
	/// <summary>
	/// A background agent that performs per-track streaming for playback.
	/// </summary>
	public class AssemblyAudioStreamingAgent : AudioStreamingAgent
	{
		/// <summary>
		/// Called when a new track requires audio decoding
		/// (typically because it is about to start playing)
		/// </summary>
		/// <param name="track">
		/// The track that needs audio streaming
		/// </param>
		/// <param name="streamer">
		/// The AudioStreamer object to which a MediaStreamSource should be
		/// attached to commence playback
		/// </param>
		/// <remarks>
		/// To invoke this method for a track set the Source parameter of the AudioTrack to null
		/// before setting  into the Track property of the BackgroundAudioPlayer instance
		/// property set to true;
		/// otherwise it is assumed that the system will perform all streaming
		/// and decoding
		/// </remarks>
		protected override void OnBeginStreaming(AudioTrack track, AudioStreamer streamer)
		{
			IsolatedStorageUtility utility = new IsolatedStorageUtility();
			Uri uri = new Uri(track.Tag, UriKind.Relative);
			Stream stream = utility.GetApplicationResourceStream(uri);
			Mp3MediaStreamSource mediaStreamSource = new Mp3MediaStreamSource(stream, stream.Length);
			streamer.SetSource(mediaStreamSource);

			//NotifyComplete(); /* Not to be called until the track is complete. */

			/* The following shows how you might retrieve the resource using a remote URI. */
			//			HttpWebRequest request = WebRequest.CreateHttp("http://localhost:4864/Audio.mp3");
			//			request.AllowReadStreamBuffering = true;
			//			request.BeginGetResponse(asyncResult =>
			//				{
			//					HttpWebResponse response = request.EndGetResponse(asyncResult) as HttpWebResponse;
			//					if (response != null)
			//					{
			//						Stream stream = response.GetResponseStream();
			//						mediaStreamSource = new Mp3MediaStreamSource(stream, response.ContentLength);
			//						streamer.SetSource(mediaStreamSource);
			//					}
			//				}, null);
		}

		/// <summary>
		/// Called when the agent request is getting cancelled
		/// </summary>
		protected override void OnCancel()
		{
			base.OnCancel();
		}
	}
}