﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;

using DanielVaughan.Data;

using Microsoft.Phone.BackgroundAudio;

namespace DanielVaughan.WindowsPhone7Unleashed.BackgroundAudio
{
	public class BackgroundAudioPlayerAgent : AudioPlayerAgent
	{
		static int trackNumber;
		Timer timer;

		static readonly List<AudioTrack> playList
			= new List<AudioTrack>
			  	{
			  		new AudioTrack(new Uri("AudioFiles/Audio01.wma", UriKind.Relative),
			  		               "Ringtone 1",			  // title
			  		               "Windows Phone",           // artist
			  		               "Windows Phone Ringtones", // album
			  		               null),
			  		new AudioTrack(new Uri("AudioFiles/Audio02.wma", UriKind.Relative),
			  		               "Ringtone 2",
			  		               "Windows Phone",
			  		               "Windows Phone Ringtones", 
			  		               null, null, EnabledPlayerControls.SkipNext),
			  		new AudioTrack(new Uri("AudioFiles/Audio03.wma", UriKind.Relative),
			  		               "Ringtone 3",
			  		               "Windows Phone",
			  		               "Windows Phone Ringtones",
			  		               null),
					new AudioTrack(null, /* Uri is null so that it is sent 
										  * to the AudioStreamingAgent 
										  * called AssemblyAudioStreamingAgent. */
			  						"Assembly Track!",
			  						"Acquired using AudioStreamingAgent",
			  						"Unleashed",
			  						null, "AudioFiles/Audio04.mp3", EnabledPlayerControls.All),
			  		/* A remote URI. */
			  		new AudioTrack(new Uri("http://traffic.libsyn.com/wpradio/WPRadio_29.mp3", UriKind.Absolute),
			  						"Episode 29",
			  						"Windows Phone Radio",
			  						"Windows Phone Radio Podcast",
			  						null)
			  	};

		internal static void CopyAudioToIsolatedStorage(IIsolatedStorageUtility isolatedStorageUtility)
		{
			ArgumentValidator.AssertNotNull(isolatedStorageUtility, "isolatedStorageUtility");

			/* If a file exists, do not copy. */
			if (isolatedStorageUtility.FileExists("AudioFiles/Audio01.wma"))
			{
				return;
			}

			/* Note that Audio04.mp3 is not copied because it is played directly 
			 * from the assembly using a custom AudioStreamingAgent called AssemblyAudioStreamingAgent. */
			string[] files = new[] { "Audio01.wma", "Audio02.wma", "Audio03.wma" };

			var sourceToDestinations = from file in files
									   let path = "AudioFiles/" + file
									   select new KeyValuePair<string, string>(path, path);

			isolatedStorageUtility.CopyApplicationResourcesToIsolatedStorage(sourceToDestinations);
		}
		
		protected override void OnPlayStateChanged(
			BackgroundAudioPlayer player, AudioTrack track, PlayState playState)
		{
			switch (playState)
			{
				case PlayState.TrackReady:
					/* The track to play is set in the PlayTrack method. */
					player.Play();
					break;

				case PlayState.TrackEnded:
					PlayNextTrack(player);
					break;
			}

			NotifyComplete();
		}

		protected override void OnUserAction(
			BackgroundAudioPlayer player, AudioTrack track, UserAction action,
			object param)
		{
			//			if (timer == null)
			//			{
			//				timer = new Timer(delegate
			//				                  	{
			//										ShellToast toast = new ShellToast
			//										{
			//											Title = "Unleashed",
			//											Content = "audio agent "
			//										};
			//										toast.Show();
			//				                  	}, null, 1000, 15000);
			//			}

			/* User actions can be initiated from the main application 
			 * or from the UVC. */
			switch (action)
			{
				case UserAction.Play:
					PlayTrack(player);
					break;

				case UserAction.Pause:
					player.Pause();
					break;

				case UserAction.SkipPrevious:
					PlayPreviousTrack(player);
					break;

				case UserAction.SkipNext:
					PlayNextTrack(player);
					break;

				case UserAction.FastForward:
					player.FastForward();
					break;
					
				case UserAction.Rewind:
					player.Rewind();
					break;

				case UserAction.Stop:
					player.Stop();
					break; 
			}

			NotifyComplete();
		}

		protected override void OnError(
			BackgroundAudioPlayer player, AudioTrack track, Exception error, bool isFatal)
		{
			//TODO: Add code to handle specific error conditions
			Console.WriteLine("OnError: isFatal=" + isFatal);

			if (isFatal)
			{
				Abort();
			}
			else
			{
				NotifyComplete();
			}
		}

		protected override void OnCancel()
		{
			//TODO: Add code to handle specific cancel conditions

			NotifyComplete();
		}

		void PlayNextTrack(BackgroundAudioPlayer player)
		{
			if (++trackNumber >= playList.Count)
			{
				trackNumber = 0;
			}

			PlayTrack(player);
		}

		void PlayPreviousTrack(BackgroundAudioPlayer player)
		{
			if (--trackNumber < 0)
			{
				trackNumber = playList.Count - 1;
			}

			PlayTrack(player);
		}

		void PlayTrack(BackgroundAudioPlayer player)
		{
			if (player.PlayerState == PlayState.Paused)
			{
				player.Play();
			}
			else
			{
				player.Track = playList[trackNumber];
			}
		}
	}
}
