﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium.

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Calcium.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-10-21 15:34:42Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System.Windows;
using System.Windows.Input;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public static class Commanding
	{
		#region Command Property

		public static DependencyProperty CommandProperty
			= DependencyProperty.RegisterAttached(
				"Command",
				typeof(ICommand),
				typeof(Commanding),
				new PropertyMetadata(OnCommandChanged));

		public static void SetCommand(DependencyObject obj, ICommand propertyValue)
		{
			obj.SetValue(CommandProperty, propertyValue);
		}

		public static ICommand GetCommand(DependencyObject obj)
		{
			return (ICommand)obj.GetValue(CommandProperty);
		}

		static void OnCommandChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			FrameworkElement element = d as FrameworkElement;
			if (element != null)
			{
				CommandBehavior behavior = GetBehavior(element);
				var command = e.NewValue as ICommand;
				behavior.Command = command;
				if (command != null && string.IsNullOrEmpty(behavior.EventName))
				{
					var eventCommand = e.NewValue as IEventCommand;
					string eventName = eventCommand != null
								&& !string.IsNullOrEmpty(eventCommand.EventName)
									? eventCommand.EventName : "Click";
					behavior.EventName = eventName;
				}
			}
		}

		#endregion

		#region Command Parameter

		public static readonly DependencyProperty CommandParameterProperty
			= DependencyProperty.RegisterAttached(
				"CommandParameter",
				typeof(object),
				typeof(Commanding),
				new PropertyMetadata(OnCommandParameterChanged));

		static void OnCommandParameterChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var frameworkElement = d as FrameworkElement;
			if (frameworkElement != null)
			{
				CommandBehavior behavior = GetBehavior(frameworkElement);
				behavior.CommandParameter = e.NewValue;
			}
		}

		public static void SetCommandParameter(
			DependencyObject obj, object propertyValue)
		{
			obj.SetValue(CommandParameterProperty, propertyValue);
		}

		public static object GetCommandParameter(DependencyObject obj)
		{
			return obj.GetValue(CommandParameterProperty);
		}

		#endregion

		#region Event Property

		public static DependencyProperty EventProperty
			= DependencyProperty.RegisterAttached(
				"Event",
				typeof(string),
				typeof(Commanding),
				new PropertyMetadata(OnEventChanged));

		public static void SetEvent(DependencyObject obj, string propertyValue)
		{
			obj.SetValue(EventProperty, propertyValue);
		}

		public static string GetEvent(DependencyObject obj)
		{
			return (string)obj.GetValue(EventProperty);
		}

		static void OnEventChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var frameworkElement = d as FrameworkElement;
			if (frameworkElement != null)
			{
				CommandBehavior behavior = GetBehavior(frameworkElement);
				var eventName = e.NewValue as string;
				behavior.EventName = eventName;
			}
		}

		#endregion

		#region CommandBehavior Property

		static readonly DependencyProperty CommandBehaviorProperty
			= DependencyProperty.RegisterAttached(
				"CommandBehavior",
				typeof(CommandBehavior),
				typeof(Commanding),
				null);

		static CommandBehavior GetBehavior(DependencyObject d)
		{
			CommandBehavior behavior
				= (CommandBehavior)d.GetValue(CommandBehaviorProperty);

			if (behavior == null)
			{
				behavior = new CommandBehavior((FrameworkElement)d);
				d.SetValue(CommandBehaviorProperty, behavior);
			}

			return behavior;
		}

		#endregion

	}
}