﻿/* The bulk of this code comes from John Gallardo
 * URL: http://blogs.msdn.com/b/jgalla/archive/2011/05/27/datacontext-log-and-windows-phone.aspx
 */
using System.Diagnostics;
using System.IO;
using System.Text;

namespace DanielVaughan.Data
{
	public class DebugStreamWriter : TextWriter
	{
		readonly int bufferSize = 256;
		readonly StringBuilder stringBuilder;

		public DebugStreamWriter(int bufferSize = 256)
		{
			this.bufferSize = ArgumentValidator.AssertGreaterThan(0, bufferSize, "bufferSize");
			stringBuilder = new StringBuilder(bufferSize);
		}

		public override Encoding Encoding
		{
			get
			{
				return Encoding.UTF8;
			}
		}

		#region StreamWriter Overrides

		public override void Write(char value)
		{
			stringBuilder.Append(value);
			if (stringBuilder.Length >= bufferSize)
			{
				Flush();
			}
		}

		public override void WriteLine(string value)
		{
			Flush();

			using (var reader = new StringReader(value))
			{
				string line;
				while ((line = reader.ReadLine()) != null)
				{
					Debug.WriteLine(line);
				}
			}
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				Flush();
			}
		}

		public override void Flush()
		{
			if (stringBuilder.Length > 0)
			{
				Debug.WriteLine(stringBuilder);
				stringBuilder.Clear();
			}
		}

		#endregion
	}
}