﻿using Microsoft.Phone.Info;

namespace DanielVaughan
{
	public interface IDeviceProperties
	{
		string WindowsLiveAnonymousId { get; }
	}

	public class DeviceProperties : IDeviceProperties
	{
		/// <summary>
		/// Gets the windows live anonymous id.
		/// This method requires ID_CAP_IDENTITY_USER
		//  to be present in the capabilities of the WMAppManifest.
		/// </summary>
		/// <returns>The string id for the user.</returns>
		static string GetWindowsLiveAnonymousId()
		{
			const int idLength = 32;
			const int idOffset = 2;

			string result = string.Empty;
			object id;
			if (UserExtendedProperties.TryGetValue("ANID", out id))
			{
				string idString = id != null ? id.ToString() : null;
				if (idString != null && idString.Length >= (idLength + idOffset))
				{
					result = idString.Substring(idOffset, idLength);
				}
			}

			return result;
		}

		string windowsLiveAnonymousId;

		public string WindowsLiveAnonymousId
		{
			get
			{
				return windowsLiveAnonymousId
				       ?? (windowsLiveAnonymousId = GetWindowsLiveAnonymousId());
			}
		}
	}
}
