﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2011-01-29 11:42:56Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Net.NetworkInformation;
using System.Threading;
using System.Windows;

using Microsoft.Phone.Net.NetworkInformation;

using NetworkInterface = Microsoft.Phone.Net.NetworkInformation.NetworkInterface;
using Microsoft.Phone.Reactive;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class NetworkConnectionMonitor : INetworkConnectionMonitor
	{
		const int sampleRateMs = 1000;
		IDisposable subscription;

		public event EventHandler<EventArgs> NetworkConnectionChanged;
		public NetworkConnectionType NetworkConnectionType  { get; private set; }

		public bool Connected
		{
			get
			{
				return NetworkConnectionType != NetworkConnectionType.None;
				//return System.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable();
			}
		}

		public NetworkConnectionMonitor()
		{
			Update();

			var observable 
				= Observable.FromEvent<NetworkAddressChangedEventHandler, EventArgs>(
					handler => new NetworkAddressChangedEventHandler(handler),
					handler => NetworkChange.NetworkAddressChanged += handler,
					handler => NetworkChange.NetworkAddressChanged -= handler);

			IObservable<IEvent<EventArgs>> sampler 
				= observable.Sample(TimeSpan.FromMilliseconds(sampleRateMs));

			subscription = sampler.ObserveOn(Scheduler.ThreadPool).Subscribe(
																args => Update());
		}
		
		void Update()
		{
			switch (NetworkInterface.NetworkInterfaceType)
			{
				case NetworkInterfaceType.None:
					NetworkConnectionType = NetworkConnectionType.None;
					break;
				case NetworkInterfaceType.MobileBroadbandCdma:
				case NetworkInterfaceType.MobileBroadbandGsm:
					NetworkConnectionType = NetworkConnectionType.MobileBroadband;
					break;
				/* These values do not apply to Windows Phone. */
				case NetworkInterfaceType.AsymmetricDsl:
				case NetworkInterfaceType.Atm:
				case NetworkInterfaceType.BasicIsdn:
				case NetworkInterfaceType.Ethernet3Megabit:
				case NetworkInterfaceType.FastEthernetFx:
				case NetworkInterfaceType.FastEthernetT:
				case NetworkInterfaceType.Fddi:
				case NetworkInterfaceType.GenericModem:
				case NetworkInterfaceType.GigabitEthernet:
				case NetworkInterfaceType.HighPerformanceSerialBus:
				case NetworkInterfaceType.IPOverAtm:
				case NetworkInterfaceType.Isdn:
				case NetworkInterfaceType.Loopback:
				case NetworkInterfaceType.MultiRateSymmetricDsl:
				case NetworkInterfaceType.Ppp:
				case NetworkInterfaceType.PrimaryIsdn:
				case NetworkInterfaceType.RateAdaptDsl:
				case NetworkInterfaceType.Slip:
				case NetworkInterfaceType.SymmetricDsl:
				case NetworkInterfaceType.TokenRing:
				case NetworkInterfaceType.Tunnel:
				case NetworkInterfaceType.Unknown:
				case NetworkInterfaceType.VeryHighSpeedDsl:
				/* Phone values */
				case NetworkInterfaceType.Ethernet:
				case NetworkInterfaceType.Wireless80211:
				default:
					NetworkConnectionType = NetworkConnectionType.Lan;
					break;
			}

			Deployment.Current.Dispatcher.BeginInvoke(new Action(
				() => NetworkConnectionChanged.Raise(this, EventArgs.Empty)));
		}
	}
}
