﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium.

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Calcium.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2011-02-11 11:53:11Z</CreationDate>
</File>
*/
#endregion

using System;
using System.Linq.Expressions;
using System.Reflection;

namespace DanielVaughan.WindowsPhone7Unleashed.Reflection
{
	public static class PropertyUtility
	{
		public static Func<TProperty> CreateGetter<TProperty>(
			this PropertyInfo propertyInfo, object owner)
		{
			ArgumentValidator.AssertNotNull(propertyInfo, "propertyInfo");
			ArgumentValidator.AssertNotNull(owner, "owner");

			Type getterType = Expression.GetFuncType(
			                    new[] { propertyInfo.PropertyType });
			//Type getterType = typeof(Func<>).MakeGenericType(propertyInfo.PropertyType);

			object getter = Delegate.CreateDelegate(
								getterType, owner, propertyInfo.GetGetMethod());

			return (Func<TProperty>)getter;
		}

		public static TDelegate CreateDelegate<TDelegate>(
			this PropertyInfo propertyInfo, 
			object owner)
		{
			ArgumentValidator.AssertNotNull(propertyInfo, "propertyInfo");
			ArgumentValidator.AssertNotNull(owner, "owner");

			object getter = Delegate.CreateDelegate(
								typeof(TDelegate), owner, propertyInfo.GetGetMethod());

			return (TDelegate)getter;
		}

		public static Action<TProperty> CreateSetter<TProperty>(
			this PropertyInfo propertyInfo, object owner)
		{
			ArgumentValidator.AssertNotNull(propertyInfo, "propertyInfo");
			ArgumentValidator.AssertNotNull(owner, "owner");

			var propertyType = propertyInfo.PropertyType;
			var setterType = Expression.GetActionType(new[] { propertyType });

			Delegate setter = Delegate.CreateDelegate(
						setterType, owner, propertyInfo.GetSetMethod());

			return (Action<TProperty>)setter;
		}

		public static PropertyInfo GetPropertyInfo<T>(Expression<Func<T>> expression)
		{
			var memberExpression = expression.Body as MemberExpression;
			if (memberExpression == null)
			{
				throw new ArgumentException(
					"MemberExpression expected.", "expression");
			}

			if (memberExpression.Member == null)
			{
				throw new ArgumentException("Member should not be null.");
			}

			if (memberExpression.Member.MemberType != MemberTypes.Property)
			{
				throw new ArgumentException("Property expected.", "expression");
			}

			PropertyInfo propertyInfo = (PropertyInfo)memberExpression.Member;
			return propertyInfo;
		}
	}
}
