﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2009, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
	This file is part of Calcium.

	Calcium is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Calcium is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with Calcium.  If not, see http://www.gnu.org/licenses/.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2009-04-25 14:43:57Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Windows;

namespace DanielVaughan.Services.Implementation
{
	public static class MessageBoxEnumTranslator
	{
		public static MessageBoxButton TranslateToMessageBoxButton(this MessageButton messageBoxButton)
		{
			return Translate(messageBoxButton);
		}

#if SILVERLIGHT
		public static MessageBoxButton Translate(MessageButton messageButton)
		{
			switch (messageButton)
			{
				case MessageButton.Ok:
					return MessageBoxButton.OK;
				case MessageButton.YesNo:
				case MessageButton.OkCancel:
				case MessageButton.YesNoCancel:
					return MessageBoxButton.OKCancel;
			}
			throw new InvalidOperationException("Unknown button type: " + messageButton);
		}
#else
		public static MessageBoxButton Translate(MessageButton button)
		{
			switch (button)
			{
				case MessageButton.Ok:
					return MessageBoxButton.OK;
				case MessageButton.OkCancel:
					return MessageBoxButton.OKCancel;
				case MessageButton.YesNo:
					return MessageBoxButton.YesNo;
				case MessageButton.YesNoCancel:
					return MessageBoxButton.YesNoCancel;
			}
			throw new InvalidOperationException("Unknown button type: " + button);
		}

				public static MessageBoxImage TranslateToMessageBoxButton(this MessageImage image)
		{
			return Translate(image);
		}

		public static MessageBoxImage Translate(MessageImage image)
		{
			switch (image)
			{
				case MessageImage.Asterisk:
					return MessageBoxImage.Asterisk;
				case MessageImage.Error:
					return MessageBoxImage.Error;
				case MessageImage.Exclamation:
					return MessageBoxImage.Exclamation;
				case MessageImage.Hand:
					return MessageBoxImage.Hand;
				case MessageImage.Information:
					return MessageBoxImage.Information;
				case MessageImage.None:
					return MessageBoxImage.None;
				case MessageImage.Question:
					return MessageBoxImage.Question;
				case MessageImage.Stop:
					return MessageBoxImage.Stop;
				case MessageImage.Warning:
					return MessageBoxImage.Warning;
			}
			throw new InvalidOperationException("Unknown image type: " + image);
		}
#endif
		public static MessageBoxResult TranslateToMessageBoxResult(this MessageResult messageResult)
		{
			return Translate(messageResult);
		}

		public static MessageBoxResult Translate(MessageResult messageResult)
		{
			switch (messageResult)
			{
				case MessageResult.Cancel:
					return MessageBoxResult.Cancel;
				case MessageResult.No:
					return MessageBoxResult.No;
				case MessageResult.None:
					return MessageBoxResult.None;
				case MessageResult.Ok:
					return MessageBoxResult.OK;
				case MessageResult.Yes:
					return MessageBoxResult.Yes;
			}
			throw new InvalidOperationException("Unknown messageResult type: " + messageResult);
		}

		public static MessageResult TranslateToMessageBoxResult(this MessageBoxResult messageBoxResult)
		{
			return Translate(messageBoxResult);
		}

		public static MessageResult Translate(MessageBoxResult messageBoxResult)
		{
			switch (messageBoxResult)
			{
				case MessageBoxResult.Cancel:
					return MessageResult.Cancel;
				case MessageBoxResult.No:
					return MessageResult.No;
				case MessageBoxResult.None:
					return MessageResult.None;
				case MessageBoxResult.OK:
					return MessageResult.Ok;
				case MessageBoxResult.Yes:
					return MessageResult.Yes;
			}
			throw new InvalidOperationException("Unknown messageBoxResult type: " + messageBoxResult);
		}
	}
}
