﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-08-18 18:36:28Z</CreationDate>
</File>
*/
#endregion


using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;

using DanielVaughan.InversionOfControl;

namespace DanielVaughan.Services.Implementation
{
	public class NavigationServiceWrapper : INavigationService
	{
		readonly NavigationService navigationService;

		public NavigationServiceWrapper(NavigationService navigationService)
		{
			this.navigationService = ArgumentValidator.AssertNotNull(navigationService, "navigationService");
		}

		public void GoBack()
		{
			navigationService.GoBack();
		}

		public void Navigate(Uri source)
		{
			ArgumentValidator.AssertNotNull(source, "source");
			navigationService.Navigate(source);
		}

		public void Navigate(string relativeUrl)
		{
			ArgumentValidator.AssertNotNull(relativeUrl, "relativeUrl");
			navigationService.Navigate(new Uri(relativeUrl, UriKind.Relative));
		}
	}

	public class FrameNavigationService : INavigationService
	{
		Frame frameUseProperty;

		Frame Frame
		{
			get
			{
				if (frameUseProperty == null)
				{
					frameUseProperty = (Frame)Application.Current.RootVisual;
				}
				return frameUseProperty;
			}
		}

		public FrameNavigationService(Frame frame)
		{
			frameUseProperty = ArgumentValidator.AssertNotNull(frame, "frame");
		}

		[InjectDependencies]
		public FrameNavigationService()
		{
			/* Intentionally left blank. */
		}

		public void GoBack()
		{
			Frame.GoBack();
		}

		public void Navigate(Uri source)
		{
			ArgumentValidator.AssertNotNull(source, "source");
			Frame.Navigate(source);
		}

		public void Navigate(string relativeUrl)
		{
			ArgumentValidator.AssertNotNull(relativeUrl, "relativeUrl");
			Frame.Navigate(new Uri(relativeUrl, UriKind.Relative));
		}
	}
}
