﻿using System;
using System.IO.IsolatedStorage;
using System.Windows;

using DanielVaughan.ComponentModel;

using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

namespace DanielVaughan.Shell
{
    public class LockScreenManager : NotifyPropertyChangeBase, ILockScreenManager
    {
        static readonly string promptedKey = "UserPromptedToAllowRunningUnderLockScreen";
        static readonly string runningEnabledKey = "RunningUnderLockScreenEnabled";

        LockScreenManager()
        {
            IsolatedStorageSettings settings = IsolatedStorageSettings.ApplicationSettings;
            bool prompted;
            if (settings.TryGetValue(promptedKey, out prompted))
            {
                UserPrompted = prompted;
            }

            bool enabledValue;
            if (settings.TryGetValue(runningEnabledKey, out enabledValue))
            {
                RunningUnderLockScreenEnabled = enabledValue;
            }

            var frame = (PhoneApplicationFrame)Application.Current.RootVisual;
            frame.Obscured += (o, args) => RunningUnderLockScreen = args.IsLocked;
            frame.Unobscured += (o, args) => RunningUnderLockScreen = false;
        }
        
        bool runningUnderLockScreen;

        public bool RunningUnderLockScreen
        {
            get
            {
                return runningUnderLockScreen;
            }
            private set
            {
                Assign(() => RunningUnderLockScreen, ref runningUnderLockScreen, value);
            }
        }

        bool runningUnderLockScreenEnabled;

        public bool RunningUnderLockScreenEnabled
        {
            get
            {
                return runningUnderLockScreenEnabled;
            }
            set
            {
                AssignmentResult result = Assign(() => RunningUnderLockScreenEnabled, 
                                ref runningUnderLockScreenEnabled, value);

                if (result == AssignmentResult.Success)
                {
                    if (runningUnderLockScreenEnabled)
                    {
                        PhoneApplicationService.Current.ApplicationIdleDetectionMode 
                                                            = IdleDetectionMode.Disabled;
                    }
                    /* Idle detection mode cannot be enabled until the application is restarted. */

                    SaveSetting(runningEnabledKey, runningUnderLockScreenEnabled);
                }
            }
        }

        bool userPrompted;

        public bool UserPrompted
        {
            get
            {
                return userPrompted;
            }
            set
            {
                if (Assign(() => UserPrompted, ref userPrompted, value) == AssignmentResult.Success)
                {
                    SaveSetting(promptedKey, value);
                }
            }
        }

        static readonly LockScreenManager instance = new LockScreenManager();

        public static LockScreenManager Instance
        {
            get
            {
                return instance;
            }
        }

        void SaveSetting(string key, object value)
        {
            IsolatedStorageSettings settings = IsolatedStorageSettings.ApplicationSettings;
            settings[key] = value;
            settings.Save();
        } 
    }
}
