﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-09-21 17:30:06Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System.Windows;
using System.Windows.Data;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class DataContextChangedListener
	{
		static readonly DependencyProperty DataContextProperty
			= DependencyProperty.RegisterAttached(
				"DataContextProperty", 
				typeof(object),
				typeof(FrameworkElement),
				new PropertyMetadata(HandleDataContextChanged));

		static readonly DependencyProperty HandlersProperty
			= DependencyProperty.RegisterAttached(
				"HandlersProperty",
				typeof(PropertyChangedCallback),
				typeof(FrameworkElement),
				new PropertyMetadata(
					(object)((PropertyChangedCallback)(delegate { }))));

		public static void Subscribe(
			FrameworkElement element, PropertyChangedCallback handler)
		{
			ArgumentValidator.AssertNotNull(element, "element");
			ArgumentValidator.AssertNotNull(handler, "handler");
			
			PropertyChangedCallback handlers 
				= (PropertyChangedCallback)element.GetValue(HandlersProperty);

			handlers += handler;

			element.SetValue(HandlersProperty, handlers);

			if (element.GetBindingExpression(DataContextProperty) == null)
			{
				element.SetBinding(DataContextProperty, new Binding());
			}
		}

		public static void Unsubscribe(
			FrameworkElement element, PropertyChangedCallback handler)
		{
			ArgumentValidator.AssertNotNull(element, "element");
			ArgumentValidator.AssertNotNull(handler, "handler");

			PropertyChangedCallback handlers 
				= (PropertyChangedCallback)element.GetValue(HandlersProperty);

			handlers -= handler;
			element.SetValue(HandlersProperty, handlers);
		}
		
		static void HandleDataContextChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			FrameworkElement element = (FrameworkElement)d;
			PropertyChangedCallback handlers 
				= (PropertyChangedCallback)element.GetValue(HandlersProperty);

			PropertyChangedCallback tempEvent = handlers;
			if (tempEvent != null)
			{
				tempEvent(d, e);
			}
		}
	}
}