﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-09-20 20:05:58Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;

namespace DanielVaughan.WindowsPhone7Unleashed.InputValidation
{
	/// <summary>
	/// Provides information about an invalid member.
	/// </summary>
	public class DataValidationError
	{
		public int Id { get; set; }

		string errorMessage;

		/// <summary>
		/// Gets or sets the error message that is displayed to the user.
		/// </summary>
		/// <value>The error message.</value>
		public string ErrorMessage
		{
			get
			{
//				var stringParserService = UnityLocator.GetInstance<IStringParserService>();
//				return stringParserService.Parse(errorMessage);
				return errorMessage;
			}
			set
			{
				errorMessage = value;
			}
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="DataValidationError"/> class.
		/// </summary>
		public DataValidationError()
		{
			/* Intentionally left blank. */
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="DataValidationError"/> class.
		/// </summary>
		/// <param name="id">The id.</param>
		/// <param name="errorMessage">The error message.</param>
		/// <exception cref="ArgumentNullException">
		/// Occurs if the specified error message is <c>null</c>.</exception>
		public DataValidationError(int id, string errorMessage)
		{
			Id = id;
			this.errorMessage 
				= ArgumentValidator.AssertNotNull(errorMessage, "errorMessage");
		}

		public override string ToString()
		{
			return ErrorMessage;
		}
	}
}