﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-10-09 15:55:08Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

using DanielVaughan.Windows;
using DanielVaughan.WindowsPhone7Unleashed.InputValidation;

namespace DanielVaughan.WindowsPhone7Unleashed.Controls
{
	public partial class ValidationSummary : UserControl
	{
		readonly ObservableCollection<object> errors 
						= new ObservableCollection<object>();

		readonly Dictionary<string, List<object>> errorLookup 
			                    = new Dictionary<string, List<object>>();
		const int lineHeight = 30;

		public IEnumerable<object> Errors
		{
			get
			{
				return errors;
			}
		}

		public ValidationSummary()
		{
			InitializeComponent();
			Loaded += HandleLoaded;
		}

	    bool loaded;

		void HandleLoaded(object sender, System.Windows.RoutedEventArgs e)
		{
            if (loaded)
            {
                return;
            }
		    loaded = true;

			Page page = this.GetVisualAncestors<Page>().FirstOrDefault();

			if (page == null)
			{
				throw new InvalidOperationException(
					            "Unable to locate parent page.");
			}
			page.BindingValidationError += HandleBindingValidationError;

			DataContextChangedListener.Subscribe(this, HandleDataContextChanged);
		}

        /// <summary>
        /// Performs property setter validation.
        /// </summary>
        void HandleBindingValidationError(object sender, ValidationErrorEventArgs e)
        {
            string content = e.Error.ErrorContent.ToString();

            if (e.Action == ValidationErrorEventAction.Added)
            {
                AddError(content);
            }
            else
            {
                RemoveError(content);
            }
        }

		void HandleDataContextChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			/* Unsubscribe to previous notifier. */
			INotifyDataErrorInfo oldNotifier = e.OldValue as INotifyDataErrorInfo;
			if (oldNotifier != null)
			{
				oldNotifier.ErrorsChanged -= HandleErrorsChanged;
			}

			/* When the DataContext is an INotifyDataErrorInfo, 
			 * monitor it for errors. */
			INotifyDataErrorInfo notifier = DataContext as INotifyDataErrorInfo;
			if (notifier != null)
			{
				notifier.ErrorsChanged += HandleErrorsChanged;
			}
		}

        /// <summary>
        /// Performs Silverlight 4 input validation.
        /// </summary>
		void HandleErrorsChanged(object sender, DataErrorsChangedEventArgs args)
		{
			INotifyDataErrorInfo notifier = sender as INotifyDataErrorInfo;
			if (notifier == null)
			{
				return;
			}

			string propertyName = args.PropertyName;
			if (string.IsNullOrEmpty(propertyName))
			{
				return;
			}

			IEnumerable notifierErrors = notifier.GetErrors(args.PropertyName);

			List<object> errorList;
			if (!errorLookup.TryGetValue(propertyName, out errorList))
			{
				errorList = new List<object>();
			}

			foreach (var error in errorList)
			{
				RemoveError(error);
			}

			errorList.Clear();

			foreach (var error in notifierErrors)
			{
                AddError(error);
			}
			errorLookup[propertyName] = errorList;

			foreach (var error in errorList)
			{
				AddError(error);
			}
		}

        void AddError(object error)
        {
            string content = error.ToString();

            if (!errors.Contains(content))
            {
                errors.Add(content);
            }
            MinHeight += lineHeight;
        }

        void RemoveError(object error)
        {
            string content = error.ToString();

            errors.Remove(content);
            MinHeight -= lineHeight;
        }
	}
}
