﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-11-20 13:29:27Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Windows;

using DanielVaughan.Windows.AppBarModel;

using Microsoft.Phone.Shell;

namespace DanielVaughan.Windows
{
    public abstract class AppBarMenuItemBase : FrameworkElement, IAppBarItem,
        IApplicationBarMenuItem, IApplicationBarMenuItemProvider
    {
        public ApplicationBarMenuItem MenuItem { get; private set; }

        protected AppBarMenuItemBase()
        {
            MenuItem = new ApplicationBarMenuItem { Text = "Text" };
            MenuItem.Click += ApplicationBarMenuItemClick;
        }

        public event EventHandler Click;

        public static readonly DependencyProperty IsEnabledProperty
            = DependencyProperty.RegisterAttached(
                "IsEnabled",
                typeof(bool),
                typeof(AppBarMenuItemBase),
                new PropertyMetadata(true, OnEnabledChanged));

        static void OnEnabledChanged(DependencyObject d,
            DependencyPropertyChangedEventArgs e)
        {
            if (e.NewValue != e.OldValue)
            {
                ((AppBarMenuItemBase)d).MenuItem.IsEnabled
                    = (bool)e.NewValue;
            }
        }

        public bool IsEnabled
        {
            get
            {
                return (bool)GetValue(IsEnabledProperty);
            }
            set
            {
                SetValue(IsEnabledProperty, value);
            }
        }

        public static readonly DependencyProperty TextProperty
            = DependencyProperty.RegisterAttached(
                "Text",
                typeof(string),
                typeof(AppBarMenuItemBase),
                new PropertyMetadata(OnTextChanged));

        static void OnTextChanged(DependencyObject d,
            DependencyPropertyChangedEventArgs e)
        {
            if (e.NewValue != e.OldValue)
            {
                ((AppBarMenuItemBase)d).MenuItem.Text
                    = e.NewValue.ToString();
            }
        }

        void ApplicationBarMenuItemClick(object sender, EventArgs e)
        {
            OnClick(e);
        }

        protected virtual void OnClick(EventArgs e)
        {
            var tempEvent = Click;
            if (tempEvent != null)
            {
                tempEvent(this, e);
            }
        }

        void IAppBarItem.PerformClick()
        {
            if (!IsEnabled)
            {
                throw new InvalidOperationException("Element not enabled.");
            }
            ApplicationBarMenuItemClick(this, EventArgs.Empty);
        }

        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }
    }
}