﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-11-20 13:29:36Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Windows;
using System.Windows.Input;

using Microsoft.Phone.Shell;

namespace DanielVaughan.Windows
{
	public class AppBarIconButton : AppBarIconButtonBase
	{
		/* This handler has been promoted to a field because 
		 * of an issue with WeakReferences being cleaned up, 
		 * and GC.KeepAlive has no effect. */
		readonly EventHandler commandCanExecuteChangedHandler;

		public AppBarIconButton()
		{
			commandCanExecuteChangedHandler = Command_CanExecuteChanged;
		}

        #region Command

        public static readonly DependencyProperty CommandProperty 
			= DependencyProperty.RegisterAttached(
				"Command", 
				typeof(ICommand),
				typeof(AppBarIconButton),
				new PropertyMetadata(OnCommandChanged));

		public ICommand Command
		{
			get
			{
				return (ICommand)GetValue(CommandProperty);
			}
			set
			{
				SetValue(CommandProperty, value);
			}
		}

		static void OnCommandChanged(DependencyObject d,
			DependencyPropertyChangedEventArgs e)
		{
			var button = d as AppBarIconButton;
			if (button == null)
			{
				return;
			}

			if (e.OldValue != null)
			{
				((ICommand)e.OldValue).CanExecuteChanged
					-= button.commandCanExecuteChangedHandler;
			}

			if (e.NewValue != null)
			{
				((ICommand)e.NewValue).CanExecuteChanged
					+= button.commandCanExecuteChangedHandler;
			}
		}

		void Command_CanExecuteChanged(object sender, EventArgs e)
		{
			ICommand command = Command;
			if (command != null)
			{
				IsEnabled = command.CanExecute(CommandParameter);
			}
		}

        #endregion

        #region CommandParameter

        public static readonly DependencyProperty CommandParameterProperty 
			= DependencyProperty.RegisterAttached(
				"CommandParameter", 
				typeof(object),
				typeof(AppBarIconButton), 
				null);

		public object CommandParameter
		{
			get
			{
				return GetValue(CommandParameterProperty);
			}
			set
			{
				SetValue(CommandParameterProperty, value);
			}
		}

        #endregion

		protected override void OnClick(EventArgs e)
		{
            base.OnClick(e);

            ICommand command = Command;
            if (command != null)
            {
                command.Execute(CommandParameter);
            }
		}
	}
}