﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-12-13 13:07:00Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Windows;
using System.Windows.Input;

using DanielVaughan.Windows.AppBarModel;

using Microsoft.Phone.Shell;

namespace DanielVaughan.Windows
{
	public class AppBarToggleMenuItem : FrameworkElement, 
		IApplicationBarMenuItemProvider
	{
		public ApplicationBarMenuItem MenuItem { get; private set; }

		/* This handler is a field because there is an issue 
		 * with the handler being cleaned up when used with Prism, 
		 * and GC.KeepAlive has not effect. */
		readonly EventHandler command1CanExecuteChangedHandler;
		readonly EventHandler command2CanExecuteChangedHandler;

		public AppBarToggleMenuItem()
		{
			MenuItem = new ApplicationBarMenuItem { Text = "unassigned" };
			MenuItem.Click += ApplicationBarMenuItemClick;
			command1CanExecuteChangedHandler = Command1_CanExecuteChanged;
			command2CanExecuteChangedHandler = Command2_CanExecuteChanged;
		}

		static void UpdateToggleState(DependencyObject d)
		{
			var button = d as AppBarToggleMenuItem;
			if (button == null)
			{
				return;
			}
			button.UpdateToggleState();
		}

		void UpdateToggleState()
		{
			bool toggled = Toggled;

			UpdateText(toggled);
			UpdateCommand(toggled);
		}

		static void UpdateText(DependencyObject d)
		{
			var button = d as AppBarToggleMenuItem;
			if (button == null)
			{
				return;
			}
			button.UpdateText(button.Toggled);
		}

		void UpdateText(bool toggled)
		{
			MenuItem.Text = toggled ? Text2 ?? Text1 : Text1;
		}

		static void UpdateCommand(DependencyObject d)
		{
			var button = d as AppBarToggleMenuItem;
			if (button == null)
			{
				return;
			}
			button.UpdateCommand(button.Toggled);
		}

		void UpdateCommand(bool toggled)
		{
			if (toggled)
			{
				var command = Command2;
				if (command != null)
				{
					IsEnabled = command.CanExecute(Command2Parameter);
				}
			}
			else
			{
				var command = Command1;
				if (command != null)
				{
					IsEnabled = command.CanExecute(Command1Parameter);
				}
			}
		}

		public event EventHandler Click;

		#region Icon1Uri Dependency Property

		public static DependencyProperty Icon1UriProperty 
			= DependencyProperty.Register(
				"Icon1Uri",
				typeof(Uri),
				typeof(AppBarToggleMenuItem), 
				new PropertyMetadata(OnIcon1Changed));

		static void OnIcon1Changed(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateToggleState(d);
		}

		public Uri Icon1Uri
		{
			get
			{
				return (Uri)GetValue(Icon1UriProperty);
			}
			set
			{
				SetValue(Icon1UriProperty, value);
			}
		}

		#endregion

		#region Command 1
		
		public static readonly DependencyProperty Command1Property 
			= DependencyProperty.RegisterAttached(
				"Command1", 
				typeof(ICommand),
				typeof(AppBarToggleMenuItem),
				new PropertyMetadata(OnCommand1Changed));

		public ICommand Command1
		{
			get
			{
				return (ICommand)GetValue(Command1Property);
			}
			set
			{
				SetValue(Command1Property, value);
			}
		}

		static void OnCommand1Changed(DependencyObject d,
			DependencyPropertyChangedEventArgs e)
		{
			var button = d as AppBarToggleMenuItem;
			if (button == null)
			{
				return;
			}

			if (e.OldValue != null)
			{
				((ICommand)e.OldValue).CanExecuteChanged
					-= button.command1CanExecuteChangedHandler;
			}

			if (e.NewValue != null)
			{
				((ICommand)e.NewValue).CanExecuteChanged
					+= button.command1CanExecuteChangedHandler;
			}
		}

		void Command1_CanExecuteChanged(object sender, EventArgs e)
		{
			if (Toggled)
			{
				return;
			}

			ICommand command = Command1;
			if (command != null)
			{
				IsEnabled = command.CanExecute(Command1Parameter);
			}
		}

		public static readonly DependencyProperty Command1ParameterProperty 
			= DependencyProperty.RegisterAttached(
				"Command1Parameter", 
				typeof(object),
				typeof(AppBarToggleMenuItem), 
				new PropertyMetadata(OnCommand1ParameterChanged));

		static void OnCommand1ParameterChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateCommand(d);
		}

		public object Command1Parameter
		{
			get
			{
				return GetValue(Command1ParameterProperty);
			}
			set
			{
				SetValue(Command1ParameterProperty, value);
			}
		}
		#endregion

		#region Command 2

		public static readonly DependencyProperty Command2Property 
			= DependencyProperty.RegisterAttached(
				"Command2", 
				typeof(ICommand),
				typeof(AppBarToggleMenuItem),
				new PropertyMetadata(OnCommand2Changed));

		public ICommand Command2
		{
			get
			{
				return (ICommand)GetValue(Command2Property);
			}
			set
			{
				SetValue(Command2Property, value);
			}
		}

		static void OnCommand2Changed(DependencyObject d,
			DependencyPropertyChangedEventArgs e)
		{
			var button = d as AppBarToggleMenuItem;
			if (button == null)
			{
				return;
			}

			if (e.OldValue != null)
			{
				((ICommand)e.OldValue).CanExecuteChanged
					-= button.command2CanExecuteChangedHandler;
			}

			if (e.NewValue != null)
			{
				((ICommand)e.NewValue).CanExecuteChanged
					+= button.command2CanExecuteChangedHandler;
			}
		}

		void Command2_CanExecuteChanged(object sender, EventArgs e)
		{
			if (!Toggled)
			{
				return;
			}

			ICommand command = Command2;
			if (command != null)
			{
				IsEnabled = command.CanExecute(Command2Parameter);
			}
		}

		public static readonly DependencyProperty Command2ParameterProperty 
			= DependencyProperty.RegisterAttached(
				"Command2Parameter", 
				typeof(object),
				typeof(AppBarToggleMenuItem),
				new PropertyMetadata(OnCommand2ParameterChanged));

		static void OnCommand2ParameterChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateCommand(d);
		}

		public object Command2Parameter
		{
			get
			{
				return GetValue(Command2ParameterProperty);
			}
			set
			{
				SetValue(Command2ParameterProperty, value);
			}
		}
		#endregion

		public static readonly DependencyProperty IsEnabledProperty 
			= DependencyProperty.RegisterAttached(
				"IsEnabled", 
				typeof(bool),
				typeof(AppBarToggleMenuItem),
				new PropertyMetadata(true, OnEnabledChanged));

		public bool IsEnabled
		{
			get
			{
				return (bool)GetValue(IsEnabledProperty);
			}
			set
			{
				SetValue(IsEnabledProperty, value);
			}
		}

		static void OnEnabledChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			AppBarToggleMenuItem appBarMenuItem;
			if (e.NewValue != e.OldValue
				&& (appBarMenuItem = d as AppBarToggleMenuItem) != null
				&& appBarMenuItem.MenuItem != null)
			{
				appBarMenuItem.MenuItem.IsEnabled = (bool)e.NewValue;
			}
		}

		#region Text 1
		
		public static readonly DependencyProperty Text1Property 
			= DependencyProperty.RegisterAttached(
				"Text1", 
				typeof(string),
				typeof(AppBarToggleMenuItem),
				new PropertyMetadata(OnText1Changed));

		public string Text1
		{
			get
			{
				return (string)GetValue(Text1Property);
			}
			set
			{
				SetValue(Text1Property, value);
			}
		}

		static void OnText1Changed(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateText(d);
		}

		#endregion

		#region Text 2

		public static readonly DependencyProperty Text2Property 
			= DependencyProperty.RegisterAttached(
				"Text2", 
				typeof(string),
				typeof(AppBarToggleMenuItem),
				new PropertyMetadata(OnText2Changed));

		public string Text2
		{
			get
			{
				return (string)GetValue(Text2Property);
			}
			set
			{
				SetValue(Text2Property, value);
			}
		}

		static void OnText2Changed(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateText(d);
		}

		#endregion

		#region Toggled Dependency Property

		public static DependencyProperty ToggledProperty 
			= DependencyProperty.Register(
				"Toggled", 
				typeof(bool), 
				typeof(AppBarToggleMenuItem),
				new PropertyMetadata(false, OnToggledChanged));

		static void OnToggledChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			UpdateToggleState(d);
		}

		public bool Toggled
		{
			get
			{
				return (bool)GetValue(ToggledProperty);
			}
			set
			{
				SetValue(ToggledProperty, value);
			}
		}

		#endregion

		void ApplicationBarMenuItemClick(object sender, EventArgs e)
		{
			ICommand command;
			object commandParameter;
			if (!Toggled)
			{
				command = Command1;
				commandParameter = Command1Parameter;
			}
			else
			{
				command = Command2 ?? Command1;
				commandParameter = Command2Parameter ?? Command1Parameter;
			}

			if (command != null)
			{
				command.Execute(commandParameter);
			}
			
			OnClick(e);
		}

		protected virtual void OnClick(EventArgs e)
		{
			var tempEvent = Click;
			if (tempEvent != null)
			{
				tempEvent(this, e);
			}
		}
	}
}