﻿using System.Collections;
using System.Windows;
using System.Windows.Controls;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public static class SelectedItems
	{
		public static readonly DependencyProperty ItemsProperty
			= DependencyProperty.RegisterAttached(
				"Items",
				typeof(IList),
				typeof(SelectedItems),
				new PropertyMetadata(null, ItemsPropertyChanged));

		public static void SetItems(ListBox listBox, IList list)
		{
			listBox.SetValue(ItemsProperty, list);
		}

		public static IList GetItems(ListBox listBox)
		{
			return (IList)listBox.GetValue(ItemsProperty);
		}

		static readonly DependencyProperty SelectedItemsBehaviorProperty 
			= DependencyProperty.RegisterAttached(
				"SelectedItemsBehavior", 
				typeof(SelectedItemsBehavior),
				typeof(ListBox), null);

		static void ItemsPropertyChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var target = d as ListBox;
			if (target != null)
			{
				GetOrCreateBehavior(target, (IList)e.NewValue);
			}
		}

		static SelectedItemsBehavior GetOrCreateBehavior(
			ListBox listBox, IList list)
		{
			var behavior = (SelectedItemsBehavior)listBox.GetValue(
												SelectedItemsBehaviorProperty);
			if (behavior == null)
			{
				behavior = new SelectedItemsBehavior(listBox, list);
				listBox.SetValue(SelectedItemsBehaviorProperty, behavior);
			}

			return behavior;
		}

		class SelectedItemsBehavior
		{
			readonly ListBox listBox;
			readonly IList sourceList;

			public SelectedItemsBehavior(ListBox listBox, IList sourceList)
			{
				this.listBox = ArgumentValidator.AssertNotNull(listBox, "listBox");
				this.sourceList = ArgumentValidator.AssertNotNull(
														sourceList, "sourceList");
				this.listBox.SelectionChanged += OnSelectionChanged;
				DataContextChangedListener.Subscribe(listBox, HandleDataContextChanged);
			}

			void HandleDataContextChanged(
				DependencyObject d, DependencyPropertyChangedEventArgs e)
			{
				UpdateList();
			}

			void OnSelectionChanged(object sender, SelectionChangedEventArgs e)
			{
				UpdateList();
			}

			void UpdateList()
			{
				sourceList.Clear();

				foreach (object item in listBox.SelectedItems)
				{
					sourceList.Add(item);
				}
			}
		}
	}
}