﻿#region File and License Information
/*
<File>
	<Copyright>Copyright © 2010, Daniel Vaughan. All rights reserved.</Copyright>
	<License>
		Redistribution and use in source and binary forms, with or without
		modification, are permitted provided that the following conditions are met:
			* Redistributions of source code must retain the above copyright
			  notice, this list of conditions and the following disclaimer.
			* Redistributions in binary form must reproduce the above copyright
			  notice, this list of conditions and the following disclaimer in the
			  documentation and/or other materials provided with the distribution.
			* Neither the name of the <organization> nor the
			  names of its contributors may be used to endorse or promote products
			  derived from this software without specific prior written permission.

		THIS SOFTWARE IS PROVIDED BY Daniel Vaughan ''AS IS'' AND ANY
		EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
		WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
		DISCLAIMED. IN NO EVENT SHALL Daniel Vaughan BE LIABLE FOR ANY
		DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
		(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
		LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
		ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
		(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
		SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	</License>
	<Owner Name="Daniel Vaughan" Email="dbvaughan@gmail.com"/>
	<CreationDate>2010-09-26 14:26:11Z</CreationDate>
	<Origin>http://www.calciumsdk.com</Origin>
</File>
*/
#endregion

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

namespace DanielVaughan.Windows
{
	public class UpdateSourceTriggerExtender
	{
		public static readonly DependencyProperty UpdateSourceOnTextChanged 
			= DependencyProperty.RegisterAttached(
				"UpdateSourceOnTextChanged", typeof(bool), 
				typeof(UpdateSourceTriggerExtender),
				new PropertyMetadata(HandleUpdatePropertyChanged));

		public static bool GetUpdateSourceOnTextChanged(DependencyObject d)
		{
			return (bool)d.GetValue(UpdateSourceOnTextChanged);
		}

		public static void SetUpdateSourceOnTextChanged(
			DependencyObject d, bool value)
		{
			d.SetValue(UpdateSourceOnTextChanged, value);
		}

		static void HandleUpdatePropertyChanged(
			DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
		    TextBox textBox = d as TextBox;
		    if (textBox != null)
		    {
		        if ((bool)e.OldValue)
		        {
		            textBox.TextChanged -= HandleTextBoxTextChanged;
		            textBox.LostFocus -= HandleBoxLostFocus;
		        }

		        if ((bool)e.NewValue)
		        {
		            textBox.TextChanged += HandleTextBoxTextChanged;
		            textBox.LostFocus += HandleBoxLostFocus;
		        }
		        return;
		    }

		    PasswordBox passwordBox = d as PasswordBox;
		    if (passwordBox == null)
		    {
		        throw new Exception("UpdateSourceTrigger can only be used " 
                                        + "on a TextBox or PasswordBox.");
		    }

		    /* Wire up for password box. */
		    if ((bool)e.OldValue)
		    {
		        passwordBox.PasswordChanged -= HandlePasswordBoxTextChanged;
		        passwordBox.LostFocus -= HandleBoxLostFocus;
		    }

		    if ((bool)e.NewValue)
		    {
		        passwordBox.PasswordChanged += HandlePasswordBoxTextChanged;
		        passwordBox.LostFocus += HandleBoxLostFocus;
		    }
		}

	    static void HandlePasswordBoxTextChanged(object sender, RoutedEventArgs e)
		{
            UpdateSource((PasswordBox)sender, PasswordBox.PasswordProperty);
		}

		static void HandleTextBoxTextChanged(object sender, TextChangedEventArgs e)
		{
            UpdateSource((TextBox)sender, TextBox.TextProperty);
		}

        static void UpdateSource(
            FrameworkElement element, DependencyProperty property)
        {
            if (element == null)
            {
                return;
            }

            BindingExpression bindingExpression
                    = element.GetBindingExpression(property);
            if (bindingExpression != null)
            {
                bindingExpression.UpdateSource();
            }
        }

		static void HandleBoxLostFocus(object sender, RoutedEventArgs e)
		{
			/* This method prevents the control from being placed 
			 * into the valid state when it loses focus. */
			var control = sender as Control;
			if (control == null)
			{
				return;
			}
			bool hasError = Validation.GetHasError(control);
			if (hasError)
			{
				VisualStateManager.GoToState(control, "InvalidFocused", false);
			}
		}
	}
}