﻿using System;
using System.Windows;
using System.Windows.Controls;

namespace DanielVaughan.Windows
{
	public static class VisualStateUtility
	{
		public static readonly DependencyProperty VisualStateProperty 
			= DependencyProperty.RegisterAttached(
				"VisualState",
			    typeof(string),
			    typeof(VisualStateUtility),
			    new PropertyMetadata(HandleVisualStateChanged));

		public static string GetVisualState(DependencyObject obj)
		{
			return (string)obj.GetValue(VisualStateProperty);
		}

		public static void SetVisualState(DependencyObject obj, string value)
		{
			obj.SetValue(VisualStateProperty, value);
		}

		static void HandleVisualStateChanged(
			object sender, DependencyPropertyChangedEventArgs args)
		{
			var control = sender as Control;
			if (control != null)
			{
				string stateName = args.NewValue != null ? args.NewValue.ToString() : null;
				if (stateName != null)
				{
                    /* Call is invoked as to avoid missing 
                     * the initial state before the control has loaded. */
				    Deployment.Current.Dispatcher.BeginInvoke(
                        delegate 
                            {
					            VisualStateManager.GoToState(control, stateName, true);
                            });
				}
			}
			else
			{
				throw new ArgumentException(
					"VisualState is only supported for Controls.");
			}
		}
	}
}