﻿using System;
using System.Diagnostics;
using System.Threading;
using System.Windows;
using System.Windows.Navigation;

using DanielVaughan.InversionOfControl;
using DanielVaughan.Services;
using DanielVaughan.Services.Implementation;
using DanielVaughan.WindowsPhone7Unleashed.Launchers;

using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

using DanielVaughan.InversionOfControl.Containers.SimpleContainer;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public partial class App : Application, IServiceProvider
	{
		/* TODO: replace with you own key. */
		public const string BingMapsId = "You Bing Maps API key";

		public App()
		{
			// Global handler for uncaught exceptions. 
			// Note that exceptions thrown by ApplicationBarItem.Click will not get caught here.
			UnhandledException += Application_UnhandledException;

			/* Standard Silverlight initialization */
			InitializeComponent();

			/* Phone-specific initialization */
			InitializePhoneApplication();

			/* XNA initialization (for hybrid features such as Microphone) */
			InitializeXnaApplication();

			InitializeContainer();

			Debug.WriteLine("App starting...");

			//TiltEffect.TiltableItems.Add(typeof(MenuItem));
		}

		void InitializeContainer()
		{
			SimpleContainer container = new SimpleContainer();
			container.InitializeServiceLocator();
#if DEBUG
			Dependency.Register<ILicensingService>(new MockLicensingService
			                                       	{
			                                       		Trial = true
			                                       	});
			Dependency.Register<IMarketplaceDetailTask>(
				new MockMarketplaceDetailTask(
					() => Debug.WriteLine("Launching Marketplace Detail...")));
#else
			Dependency.Register<ILicensingService, LicensingService>();
			Dependency.Register<IMarketplaceDetailTask, 
									MarketplaceDetailTaskAdapter>();
#endif
			Dependency.Register<IMessageService, MessageService>(true);
			Dependency.Register<INavigationService, FrameNavigationService>(true);
		}

		// Code to execute when the application is launching (eg, from Start)
		// This code will not execute when the application is reactivated
		void Application_Launching(object sender, LaunchingEventArgs e)
		{
			//Thread.Sleep(10000);
			Debug.WriteLine("Application Launching");

			StateManager.Initialize();

			/* For the Twitter demo. */
			var twitterDatabaseUtility = new TwitterDatabaseUtility();
			bool wipeDatabase = Debugger.IsAttached;
			twitterDatabaseUtility.InitializeDatabase(wipeDatabase);

			//IsolatedStorageExplorer.Explorer.Start("localhost");
		}

		// Code to execute when the application is activated (brought to foreground)
		// This code will not execute when the application is first launched
		void Application_Activated(object sender, ActivatedEventArgs e)
		{
			Debug.WriteLine("Application Activated");

            if (e.IsApplicationInstancePreserved)
            {
                /* Application was placed in the dormant state. */                    
            }
            else
            {
                /* Application state should be restored manually. */
            }

			StateManager.Initialize();

			//IsolatedStorageExplorer.Explorer.RestoreFromTombstone();
		}

		// Code to execute when the application is deactivated (sent to background)
		// This code will not execute when the application is closing
		void Application_Deactivated(object sender, DeactivatedEventArgs e)
		{
			Debug.WriteLine("Application Deactivated");
			//StateManager.ShouldLoadTransientState = false;
			//			var random = new Random();
			//			int ran = random.Next(0, 1000);
			//			Debug.WriteLine(ran + "  Application Deactivated");
			//			DateTime startTime = DateTime.Now;
			//			for (int i = 0; i < 20; i++)
			//			{
			//				Thread.Sleep(5000);
			//				Debug.WriteLine(string.Format(ran + "  Slept for {0}", (DateTime.Now - startTime).TotalSeconds));
			//			}
			//
			//			Debug.WriteLine(ran + "  Finished sleeping");
		}


		// Code to execute when the application is closing (eg, user hit Back)
		// This code will not execute when the application is deactivated
		void Application_Closing(object sender, ClosingEventArgs e)
		{
			Debug.WriteLine("Application Closing");
            //			System.IO.IsolatedStorage.IsolatedStorageSettings.ApplicationSettings["Key"] = someObject;

            //			var random = new Random();
            //			int ran = random.Next(0, 1000);
            //			Debug.WriteLine(ran + "  Application Closing");
            //			DateTime startTime = DateTime.Now;
            //			for (int i = 0; i < 20; i++)
            //			{
            //				Thread.Sleep(5000);
            //				Debug.WriteLine(string.Format(ran + "  Slept for {0}", (DateTime.Now - startTime).TotalSeconds));
            //			}
            //
            //			Debug.WriteLine(ran + "  Finished sleeping");
		}

		// Code to execute if a navigation fails
		void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
		{
			if (Debugger.IsAttached)
			{
				// A navigation has failed; break into the debugger
				Debugger.Break();
			}
		}

		// Code to execute on Unhandled Exceptions
		void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
		{
			if (Debugger.IsAttached)
			{
				// An unhandled exception has occurred; break into the debugger
				Debugger.Break();
			}
		}

		#region Phone application initialization

		bool phoneApplicationInitialized;

		void InitializePhoneApplication()
		{
			if (phoneApplicationInitialized)
			{
				return;
			}

			// Create the frame but don't set it as RootVisual yet; this allows the splash
			// screen to remain active until the application is ready to render.
            //			RootFrame = new PhoneApplicationFrame();
            //			RootFrame.Navigated += CompleteInitializePhoneApplication;
            //
			            // Handle navigation failures
            //			RootFrame.NavigationFailed += RootFrame_NavigationFailed;

			// Ensure we don't initialize again
			phoneApplicationInitialized = true;
		}

		void RootFrame_Navigated(object sender, NavigationEventArgs e)
		{
//			if (!StateManager.ShouldLoadTransientState)
//			{
//				return;
//			}
//
//			var element = e.Content as FrameworkElement;
//			if (element != null)
//			{
//				IStateAware stateAware = element.DataContext as IStateAware;
//				if (stateAware != null)
//				{
//					stateAware.LoadTransientState(PhoneApplicationService.Current.State);
//				}
//			}
		}

		#endregion

		void RootFrame_Navigating(object sender, NavigatingCancelEventArgs e)
		{
            //			var frame = (PhoneApplicationFrame)RootVisual;
            //			var element = frame.Content as FrameworkElement;
            //			if (element != null)
            //			{
            //				IStateAware stateAware = element.DataContext as IStateAware;
            //				if (stateAware != null)
            //				{
            //					stateAware.SaveTransientState(PhoneApplicationService.Current.State);
            //				}
            //			}
		}

		#region XNA

		/// <summary>
		/// Provides access to a ContentManager for the application.
		/// </summary>
		public ContentManager ContentManager { get; private set; }

		// Performs initialization of the XNA types required for the application.
		void InitializeXnaApplication()
		{
			// Create the ContentManager so the application can load precompiled assets
			ContentManager = new ContentManager(this, "Content");

			// Create a GameTimer to pump the XNA FrameworkDispatcher
			GameTimer frameworkDispatcherTimer = new GameTimer();
			frameworkDispatcherTimer.FrameAction += FrameworkDispatcherFrameAction;
			frameworkDispatcherTimer.Start();
		}

		// An event handler that pumps the FrameworkDispatcher each frame.
		// FrameworkDispatcher is required for a lot of the XNA events and
		// for certain functionality such as SoundEffect playback.
		void FrameworkDispatcherFrameAction(object sender, EventArgs e)
		{
			FrameworkDispatcher.Update();
		}

		#endregion

		#region IServiceProvider Members

		/// <summary>
		/// Gets a service from the ApplicationLifetimeObjects collection.
		/// </summary>
		/// <param name="serviceType">The type of service to retrieve.</param>
		/// <returns>The first item in the ApplicationLifetimeObjects collection of the requested type.</returns>
		public object GetService(Type serviceType)
		{
			// Find the first item that matches the requested type
			foreach (object item in ApplicationLifetimeObjects)
			{
				if (serviceType.IsAssignableFrom(item.GetType()))
				{
					return item;
				}
			}

			// Throw an exception if there was no matching item
			throw new InvalidOperationException(
                "No object in the ApplicationLifetimeObjects is assignable to " 
                    + serviceType);
		}

		#endregion
	}
}
