﻿using System.Device.Location;
using System.Linq;
using System.Text;
using System.Xml.Linq;

using DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Route;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	/// <summary>
	/// This class represents a Bing Route ItineraryItem, 
	/// but cleans the text value, and automatically converts the item.Location value 
	/// to a GeoCoordinate; making it compatible with the Bing Map control.
	/// </summary>
	public class MapItineraryItem
	{
		public string Text { get; private set; }
		public GeoCoordinate Location { get; private set; }
		/// <summary>
		/// Gets or sets the distance. Units are in kilometers by default.
		/// The unit can be changed by specifying the UserProfile.DistanceUnit 
		/// before making a request.
		/// </summary>
		/// <value>The distance.</value>
		public double Distance { get; private set; }

		public string DistanceString 
		{ 
			get
			{
				return Distance + " km";
			} 
		}

		public MapItineraryItem(ItineraryItem item)
		{
			ArgumentValidator.AssertNotNull(item, "item");
			Text = Extract(item.Text);
			Location = item.Location;
			Distance = item.Summary.Distance;
		}

		static string Extract(string value)
		{
			ArgumentValidator.AssertNotNull(value, "value");

			var stringBuilder = new StringBuilder();

			string cleanedXml = value.Replace("&", "&amp;");
			string validXmlText = string.Format(
				"<Root xmlns:VirtualEarth=\"http://BingMaps\">{0}</Root>", cleanedXml);

			XDocument document = XDocument.Parse(validXmlText);

			XElement childElement = document.FirstNode as XElement;
			if (childElement != null)
			{
				foreach (XNode node in childElement.Nodes())
				{
					XElement element = node as XElement;
					stringBuilder.Append(
						element != null ? element.Value : node.ToString());
					stringBuilder.Append(' ');
				}
			}

			return stringBuilder.Replace("  ", " ").ToString();
		}
	}
}
