﻿using System;

using DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Geocode;
using DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Route;

using ResponseStatusCode = DanielVaughan.WindowsPhone7Unleashed.Examples.Bing.Geocode.ResponseStatusCode;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.BingMaps.Routing
{
	public class RouteCalculationError
	{
		const string noResultsReason = "No results.";

		public string Reason { get; private set; }
		public Exception Exception { get; private set; }

		internal RouteCalculationError(string reason, Exception exception)
		{
			Reason = reason;
			Exception = exception;
		}

		internal RouteCalculationError(GeocodeCompletedEventArgs e)
		{
			ArgumentValidator.AssertNotNull(e, "e");
			if (e.Result == null 
				|| e.Result.ResponseSummary == null 
				|| string.IsNullOrEmpty(e.Result.ResponseSummary.FaultReason))
			{
				Reason = noResultsReason;
			}
			else
			{
				Reason = e.Result.ResponseSummary.FaultReason;
			}

			Exception = e.Error;
		}

		internal RouteCalculationError(CalculateRouteCompletedEventArgs e)
		{
			ArgumentValidator.AssertNotNull(e, "e");
			if (e.Result == null 
				|| e.Result.ResponseSummary == null 
				|| string.IsNullOrEmpty(e.Result.ResponseSummary.FaultReason))
			{
				Reason = noResultsReason;
			}
			else
			{
				Reason = e.Result.ResponseSummary.FaultReason;
			}

			Exception = e.Error;
		}

		public static RouteCalculationError CreateErrorIfInvalid(GeocodeCompletedEventArgs e)
		{
			if (e.Error != null 
				|| e.Result.ResponseSummary == null 
				|| e.Result.ResponseSummary.StatusCode != ResponseStatusCode.Success)
			{
				return new RouteCalculationError(e);
			}
			return null;
		}

		public static RouteCalculationError CreateErrorIfInvalid(CalculateRouteCompletedEventArgs e)
		{
			if (e.Error != null 
				|| e.Result.ResponseSummary == null 
				|| e.Result.ResponseSummary.StatusCode != Bing.Route.ResponseStatusCode.Success)
			{
				return new RouteCalculationError(e);
			}
			return null;
		}
	}
}
