﻿using System.Collections;
using System.Collections.ObjectModel;
using System.Windows.Input;

using DanielVaughan.Services;
using DanielVaughan.WindowsPhone7Unleashed.Launchers;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class ChatClientViewModel : ViewModelBase
	{
		readonly IChatService chatService;

		public ChatClientViewModel(IChatService chatService)
		{
			this.chatService = ArgumentValidator.AssertNotNull(
											chatService, "chatService");

			sendCommand = new DelegateCommand(
				delegate
					{
						if (string.IsNullOrEmpty(message))
						{
							return;
						}
						chatService.SendMessage(message);
						Message = string.Empty;
					}, 
				delegate { return !string.IsNullOrEmpty(message); });

			chatService.MessageReceived 
				+= (sender, args) => messages.Add(args.Message);

			PropertyChanged += delegate { sendCommand.RaiseCanExecuteChanged(); };

			var licensingService = Dependency.Resolve<ILicensingService>();
			buyOptionVisible = licensingService.Trial;

			buyCommand = new DelegateCommand(
				obj =>
				{
					var marketplaceDetailTaskAdapter 
						= Dependency.Resolve<IMarketplaceDetailTask>();
					marketplaceDetailTaskAdapter.Show();
				});
		}

		readonly ObservableCollection<string> messages = new ObservableCollection<string>();
/* TODO: */
//						  	{
//						  		"Hi from Windows Phone 7 Unleashed!",
//								"What's happening?",
//								"We're unit testing!",
//								"On the phone?"};

		public ObservableCollection<string> Messages
		{
			get
			{
				return messages;
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
				Assign(() => Message, ref message, value);
			}
		}

		readonly DelegateCommand sendCommand;

		public ICommand SendCommand
		{
			get
			{
				return sendCommand;
			}
		}

		bool buyOptionVisible;

		public bool BuyOptionVisible
		{
			get
			{
				return buyOptionVisible;
			}
			private set
			{
				Assign(() => BuyOptionVisible, ref buyOptionVisible, value);
			}
		}

		readonly DelegateCommand buyCommand;

		public ICommand BuyCommand
		{
			get
			{
				return buyCommand;
			}
		}
	}
}
