﻿using System;
using System.Collections.Generic;
using System.Windows.Input;

using Microsoft.Phone.UserData;

using System.Linq;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class ContactsViewModel : ViewModelBase
	{
        public ContactsViewModel() : base("search contacts")
        {
            searchCommand = new DelegateCommand(arg => SearchContacts());
        }

        Contacts contacts;

	    void SearchContacts()
	    {
            if (string.IsNullOrWhiteSpace(searchText) 
                    && (FilterKind == FilterKind.EmailAddress 
                            || FilterKind == FilterKind.PhoneNumber))
            {
                MessageService.ShowMessage("Please enter the search text.");
                return;
            }

            contacts = new Contacts();

            contacts.SearchCompleted += HandleSearchCompleted;
            contacts.SearchAsync(searchText, FilterKind, null);
	    }

	    void HandleSearchCompleted(object sender, ContactsSearchEventArgs e)
	    {
	        if (e.Results == null)
	        {
	            return;
	        }
	        Contacts = e.Results;
	    }

	    IEnumerable<Contact> contactsList = new List<Contact>();

	    public IEnumerable<Contact> Contacts
	    {
	        get
	        {
	            return contactsList;
	        }
	        private set
	        {
                Assign(() => Contacts, ref contactsList, value);
	        }
	    }

	    readonly DelegateCommand searchCommand;

		public ICommand SearchCommand
		{
			get
			{
				return searchCommand;
			}
		}

	    string searchText;

	    public string SearchText
	    {
	        get
	        {
	            return searchText;
	        }
	        set
	        {
	            Assign(() => SearchText, ref searchText, value);
	        }
	    }

        FilterKind filterKind = filterKinds.First();

	    public FilterKind FilterKind
	    {
	        get
	        {
	            return filterKind;
	        }
	        set
	        {
	            Assign(() => FilterKind, ref filterKind, value);
	        }
	    }

	    static readonly IEnumerable<FilterKind> filterKinds 
            = EnumUtility.CreateEnumValueList<FilterKind>().OrderBy(x => x.ToString());

	    public IEnumerable<FilterKind> FilterKinds
	    {
	        get
	        {
	            return filterKinds;
	        }
	    }
	}
}
