﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Reflection;
using System.Windows.Input;
using System.Windows.Media;

using DanielVaughan.Collections;

using Microsoft.Phone.Shell;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.ControlExamples
{
	public class ApplicationBarViewModel : ViewModelBase
	{
		public ApplicationBarViewModel() : base("application bar")
		{
			showMessage1Command = new DelegateCommand(
				obj =>
				{
					Message = "Command 1 Executed.";
				});

			toggleMuteCommand = new DelegateCommand(
				obj =>
				{
					Message = Muted ? "Muted" : "Unmuted";
					Muted = !Muted;
				});

			pauseCommand = new DelegateCommand(
				obj =>
				{
					Paused = true;
					Message = "Paused";
				});

			resumeCommand = new DelegateCommand(
				obj =>
				{
					Paused = false;
					Message = "Resumed";
				});

			simpleCommand = new DelegateCommand(
				parameter =>
				{
					Message = "Simple command executed with command parameter: " 
									+ parameter;
				});

			fullScreenCommand = new DelegateCommand<bool>(
				visible =>
					{
						SystemTrayVisible = ApplicationBarVisible = !visible;
					});

			colors.AddRange(GetColors());
			backgroundColor = colors[0];
			foregroundColor = colors[1];
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
				Assign(() => Message, ref message, value);
			}
		}

		readonly DelegateCommand showMessage1Command;

		public ICommand ShowMessage1Command
		{
			get
			{
				return showMessage1Command;
			}
		}

		readonly DelegateCommand toggleMuteCommand;

		public ICommand ToggleMuteCommand
		{
			get
			{
				return toggleMuteCommand;
			}
		}

		bool muted;

		public bool Muted
		{
			get
			{
				return muted;
			}
			set
			{
				Assign(() => Muted, ref muted, value);
			}
		}


		readonly DelegateCommand pauseCommand;

		public ICommand PauseCommand
		{
			get
			{
				return pauseCommand;
			}
		}

		readonly DelegateCommand resumeCommand;

		public ICommand ResumeCommand
		{
			get
			{
				return resumeCommand;
			}
		}

		bool paused;

		public bool Paused
		{
			get
			{
				return paused;
			}
			set
			{
				Assign(() => Paused, ref paused, value);
			}
		}

		readonly DelegateCommand simpleCommand;

		public ICommand SimpleCommand
		{
			get
			{
				return simpleCommand;
			}
		}

		readonly DelegateCommand<bool> fullScreenCommand;

		public ICommand FullScreenCommand
		{
			get
			{
				return fullScreenCommand;
			}
		}

		bool applicationBarVisible = true;

		public bool ApplicationBarVisible
		{
			get
			{
				return applicationBarVisible;
			}
			private set
			{
				Assign(() => ApplicationBarVisible, ref applicationBarVisible, value);
			}
		}

		bool systemTrayVisible = true;

		public bool SystemTrayVisible
		{
			get
			{
				return systemTrayVisible;
			}
			private set
			{
				Assign(() => SystemTrayVisible, ref systemTrayVisible, value);
			}
		}

		bool menuEnabled = true;

		public bool MenuEnabled
		{
			get
			{
				return menuEnabled;
			}
			set
			{
				Assign(() => MenuEnabled, ref menuEnabled, value);
			}
		}

		bool applicationBarEnabled = true;

		public bool ApplicationBarEnabled
		{
			get
			{
				return applicationBarEnabled;
			}
			set
			{
				Assign(() => ApplicationBarEnabled, ref applicationBarEnabled, value);
			}
		}

		double barOpacity = 1;

		public double BarOpacity
		{
			get
			{
				return barOpacity;
			}
			set
			{
				Assign(() => BarOpacity, ref barOpacity, value);
			}
		}

		Color backgroundColor;

		public Color BackgroundColor
		{
			get
			{
				return backgroundColor;
			}
			set
			{
				Assign(() => BackgroundColor, ref backgroundColor, value);
			}
		}

		Color foregroundColor;

		public Color ForegroundColor
		{
			get
			{
				return foregroundColor;
			}
			set
			{
				Assign(() => ForegroundColor, ref foregroundColor, value);
			}
		}

		readonly ObservableCollection<Color> colors = new ObservableCollection<Color>();

		public IEnumerable<Color> Colors
		{
			get
			{
				return colors;
			}
		}

		bool firstButtonVisible = true;

		public bool FirstButtonVisible
		{
			get
			{
				return firstButtonVisible;
			}
			set
			{
				Assign(() => FirstButtonVisible, ref firstButtonVisible, value);
			}
		}

	    ApplicationBarMode mode;

	    public ApplicationBarMode Mode
	    {
	        get
	        {
	            return mode;
	        }
	        set
	        {
	            Assign(() => Mode, ref mode, value);
	        }
	    }

		static IEnumerable<Color> GetColors()
		{
			/* Use reflection to get known colors. */
			Type colorsType = typeof(Colors);
			PropertyInfo[] propertyInfos = colorsType.GetProperties(
				BindingFlags.Public | BindingFlags.Static);

			List<Color> knownColors = new List<Color>();
			foreach (PropertyInfo propertyInfo in propertyInfos)
			{
				try
				{
					Color color = (Color)propertyInfo.GetValue(null, null);
					knownColors.Add(color);
				}
				catch (Exception ex)
				{
					continue;
				}
			}

			return knownColors;
		}

		public void AddKnownColor(Color color)
		{
			colors.Add(color);
		}
	}
}
