﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;

using DanielVaughan.ComponentModel;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class ButtonExampleViewModel : ViewModelBase
	{
		public ButtonExampleViewModel() : base("Buttons")
		{
			buttonCommand = new DelegateCommand(
				obj => Message = string.Format(
					"Command executed {0} times.", ++buttonCommandExecuteCount));

			repeatCommand = new DelegateCommand(obj => RepeatCount++);

			//selectedRadioItem = radioItems[0];
		}

		int repeatCount;

		public int RepeatCount
		{
			get
			{
				return repeatCount;
			}
			private set
			{
				Assign(() => RepeatCount, ref repeatCount, value);
			}
		}

		int buttonCommandExecuteCount;

		readonly DelegateCommand buttonCommand;

		public ICommand ButtonCommand
		{
			get
			{
				return buttonCommand;
			}
		}

		readonly DelegateCommand repeatCommand;

		public ICommand RepeatCommand
		{
			get
			{
				return repeatCommand;
			}
		}

		bool? isChecked;

		public bool? Checked
		{
			get
			{
				return isChecked;
			}
			set
			{
				Assign(() => Checked, ref isChecked, value);
			}
		}

		readonly ObservableCollection<DataItem> radioItems 
			= new ObservableCollection<DataItem>
			  	{
			  		new DataItem(0, "Item 1"),
					new DataItem(1, "Item 2"),
					new DataItem(2, "Item 3")
			  	};

		public IEnumerable<DataItem> RadioItems
		{
			get
			{
				return radioItems;
			}
		}

		DataItem selectedRadioItem;

		public DataItem SelectedRadioItem
		{
			get
			{
				return selectedRadioItem;
			}
			set
			{
				Assign(() => SelectedRadioItem, ref selectedRadioItem, value);
			}
		}

		readonly ObservableCollection<DataItem> checkBoxItems
			= new ObservableCollection<DataItem>
			  	{
			  		new DataItem(0, "Item 1"),
					new DataItem(1, "Item 2"),
					new DataItem(2, "Item 3")
			  	};

		public IEnumerable<DataItem> CheckBoxItems
		{
			get
			{
				return checkBoxItems;
			}
		}

		readonly ObservableCollection<DataItem> selectedCheckBoxItems 
			= new ObservableCollection<DataItem>();

		public ObservableCollection<DataItem> SelectedCheckBoxItems
		{
			get
			{
				return selectedCheckBoxItems;
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			private set
			{
				Assign(() => Message, ref message, value);
			}
		}	
	}

	public class DataItem : NotifyPropertyChangeBase
	{
		object itemValue;

		public object Value
		{
			get
			{
				return itemValue;
			}
			set
			{
				Assign("Value", ref itemValue, value);
			}
		}

		object text;

		public object Text
		{
			get
			{
				return text;
			}
			set
			{
				Assign("Text", ref text, value);
			}
		}	

		public DataItem(object value, object text)
		{
			itemValue = ArgumentValidator.AssertNotNull(value, "value");
			this.text = ArgumentValidator.AssertNotNull(text, "text");
		}

		public override string ToString()
		{
			return text != null ? text.ToString() : base.ToString();
		}
	}
}
