﻿
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Reflection;
using System.Threading;
using System.Windows.Input;
using System.Linq;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class ControlExamplesViewModel : ViewModelBase
	{
		readonly BackgroundWorker backgroundWorker = new BackgroundWorker();

		public ControlExamplesViewModel() : base("Controls")
		{
			backgroundWorker.WorkerReportsProgress = true;
			backgroundWorker.DoWork 
				+= new DoWorkEventHandler(backgroundWorker_DoWork);
			backgroundWorker.ProgressChanged 
				+= new ProgressChangedEventHandler(backgroundWorker_ProgressChanged);
			backgroundWorker.RunWorkerCompleted 
				+= new RunWorkerCompletedEventHandler(
					backgroundWorker_RunWorkerCompleted);

			backgroundWorker.WorkerSupportsCancellation = true;
			cancelProgressCommand = new DelegateCommand(
				obj => backgroundWorker.CancelAsync());

			backgroundWorker.RunWorkerAsync();
			Message = "BackgroundWorker performing task.";
		}

		void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			ProgressPercent = e.ProgressPercentage;
		}

		void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
		{
			BackgroundWorker worker = (BackgroundWorker)sender;
			for (int i = 0; i < 100; i++)
			{
				if (worker.CancellationPending)
				{
					e.Cancel = true;
					return;
				}
				Wait(300);
				worker.ReportProgress(i);
			}
		}

		void backgroundWorker_RunWorkerCompleted(
				object sender, RunWorkerCompletedEventArgs e)
		{
			Message = e.Cancelled 
				? "Background worker cancelled." 
				: "Background worker completed.";
		}

		int progressPercent;

		public int ProgressPercent
		{
			get
			{
				return progressPercent;
			}
			private set
			{
				Assign("ProgressPercent", ref progressPercent, value);
			}
		}

		bool cancelProgress;
		DelegateCommand cancelProgressCommand;

		public ICommand CancelProgressCommand
		{
			get
			{
				return cancelProgressCommand;
			}
		}		

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
				Assign(() => Message, ref message, value);
			}
		}	

		static readonly object waitLock = new object();

		static void Wait(int delayMs)
		{
			if (delayMs > 0)
			{
				lock (waitLock)
				{
					Monitor.Wait(waitLock, delayMs);
				}
			}
		}

		#region InputScopes

        readonly IEnumerable<InputScopeNameValue> inputScopeNameValues
                                = EnumUtility.CreateEnumValueList<InputScopeNameValue>().OrderBy(nameValue => nameValue.ToString());

		public IEnumerable<InputScopeNameValue> InputScopeNameValues
		{
			get
			{
				return inputScopeNameValues;
			}
		}

		InputScopeNameValue inputScopeNameValue = InputScopeNameValue.Default;

		public InputScopeNameValue InputScopeNameValue
		{
			get
			{
				return inputScopeNameValue;
			}
			set
			{
				Assign(() => InputScopeNameValue, ref inputScopeNameValue, value);
			}
		}

        //		static IEnumerable<TEnum> CreateEnumValueList<TEnum>()
        //		{
        //			Type enumType = typeof(TEnum);
        //			var fieldInfos = enumType.GetFields().Where(x => enumType.Equals(x.FieldType));
        //			return fieldInfos.Select(
        //				fieldInfo => (TEnum)Enum.Parse(enumType, fieldInfo.Name, true)).ToList();
        //		}
        //
        //		static IEnumerable<string> CreateEnumStringValueList<TEnum>()
        //		{
        //			Type enumType = typeof(TEnum);
        //			var fieldInfos = typeof(TEnum).GetFields().Where(x => enumType.Equals(x.FieldType));
        //			return fieldInfos.Select(fieldInfo => fieldInfo.Name).ToList();
        //		}
		#endregion

		string password;

		public string Password
		{
			get
			{
				return password;
			}
			set
			{
				Assign(() => Password, ref password, value);
			}
		}	
	}
}
