﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

using Microsoft.Phone.Controls;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public partial class DeepZoomView : PhoneApplicationPage
	{
		Point dragStartPoint;
		Point dragStartViewportOrigin;
		Point pinchLogicalStartPoint;
		double pinchStartLevel = 1;

		public DeepZoomView()
		{
			InitializeComponent();
			DataContext = new DeepZoomViewModel();

			MultiScaleImage multiScaleImage = new MultiScaleImage
			                                  	{
			                                  		Source = new DeepZoomImageTileSource(new Uri("http://www.example.com/dzc_output.xml"))
			                                  	};
		}

		double zoomLevel = 1;

		void Zoom(double level, Point point)
		{
			const double minimumZoomLevel = 0.5;
			if (level < minimumZoomLevel)
			{
				level = minimumZoomLevel;
			}

			multiScaleImage.ZoomAboutLogicalPoint(
				level / zoomLevel, point.X, point.Y);

			zoomLevel = level;
		}

		void OnPinchStarted(object sender, PinchStartedGestureEventArgs e)
		{
			pinchStartLevel = zoomLevel;
			Point pinchStartPoint = e.GetPosition(multiScaleImage);
			pinchLogicalStartPoint 
				= multiScaleImage.ElementToLogicalPoint(pinchStartPoint);
		}

		void OnPinchDelta(object sender, PinchGestureEventArgs e)
		{
			Zoom(e.DistanceRatio * pinchStartLevel, pinchLogicalStartPoint);
		}

		void OnDragStarted(object sender, DragStartedGestureEventArgs e)
		{
			dragStartPoint = e.GetPosition(multiScaleImage);
			dragStartViewportOrigin = multiScaleImage.ViewportOrigin;
		}

		void OnDragDelta(object sender, DragDeltaGestureEventArgs e)
		{
			Point touchPoint = e.GetPosition(multiScaleImage);
			double visibleSize 
				= multiScaleImage.ActualWidth / multiScaleImage.ViewportWidth;
			Point newPoint = dragStartViewportOrigin;
			newPoint.X += (dragStartPoint.X - touchPoint.X) / visibleSize;
			newPoint.Y += (dragStartPoint.Y - touchPoint.Y) / visibleSize;
			multiScaleImage.ViewportOrigin = newPoint;
		}

		void OnDoubleTap(object sender, GestureEventArgs e)
		{
			Point tapPoint = e.GetPosition(multiScaleImage);
			Point logicalPoint = multiScaleImage.ElementToLogicalPoint(tapPoint);
			Zoom(zoomLevel * 2, logicalPoint);
		}
	}
}