﻿using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class InkPresenterViewModel : ViewModelBase
	{
		readonly Stack<Stroke> undoneStrokes = new Stack<Stroke>();
		Stroke stroke;

		public InkPresenterViewModel() : base("InkPresenter")
		{
			beginStrokeCommand = new DelegateCommand<Point>(
				point =>
					{
						stroke = new Stroke();
						stroke.StylusPoints.Add(ConvertToStylusPoint(point));
						stroke.DrawingAttributes.Color = stylusColor;
						strokes.Add(stroke);
						Debug.WriteLine("<StylusPoint X=\"{0}\" Y=\"{1}\" />", point.X, point.Y);
					});

			setStrokePointCommand = new DelegateCommand<Point>(
				point =>
					{
						if (stroke != null)
						{
							stroke.StylusPoints.Add(ConvertToStylusPoint(point));
							Debug.WriteLine("<StylusPoint X=\"{0}\" Y=\"{1}\" />", point.X, point.Y);
						}
					});

			endStrokeCommand = new DelegateCommand(obj => stroke = null);

			clearCommand = new DelegateCommand(obj => strokes.Clear());

			undoCommand = new DelegateCommand(
				delegate
			        {
						if (strokes.Count > 0)
						{
							undoneStrokes.Push(strokes.Last());
							strokes.RemoveAt(strokes.Count - 1);
						}
			        });

			redoCommand = new DelegateCommand(
				delegate
					{
						if (undoneStrokes.Count > 0)
						{
							strokes.Add(undoneStrokes.Pop());
						}
					});
		}

		static StylusPoint ConvertToStylusPoint(Point position)
		{
			return new StylusPoint(position.X, position.Y);
		}

		readonly DelegateCommand undoCommand;

		public ICommand UndoCommand
		{
			get
			{
				return undoCommand;
			}
		}

		readonly DelegateCommand redoCommand;

		public ICommand RedoCommand
		{
			get
			{
				return redoCommand;
			}
		}

		readonly DelegateCommand<Point> beginStrokeCommand;

		public ICommand BeginStrokeCommand
		{
			get
			{
				return beginStrokeCommand;
			}
		}

		readonly DelegateCommand<Point> setStrokePointCommand;

		public ICommand SetStrokePointCommand
		{
			get
			{
				return setStrokePointCommand;
			}
		}

		readonly DelegateCommand endStrokeCommand;

		public ICommand EndStrokeCommand
		{
			get
			{
				return endStrokeCommand;
			}
		}

		readonly DelegateCommand clearCommand;

		public ICommand ClearCommand
		{
			get
			{
				return clearCommand;
			}
		}

		readonly StrokeCollection strokes = new StrokeCollection();

		public StrokeCollection Strokes
		{
			get
			{
				return strokes;
			}
		}

		Color stylusColor = Colors.Yellow;

		public Color StylusColor
		{
			get
			{
				return stylusColor;
			}
			set
			{
				Assign(() => StylusColor, ref stylusColor, value);
			}
		}	
	}
}