﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Net;
using System.Windows;
using System.Windows.Threading;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class MediaDownloader
	{
		public Uri Uri { get; private set; }
		public string FileName { get; private set; }
		public string IsolateStorageDirectory { get; set; }
		WebClient webClient;
		public Exception Error { get; private set; }

		public MediaDownloader(Uri uri, string isolateStorageDirectory, string isolatedStoragefileName)
		{
			Uri = ArgumentValidator.AssertNotNull(uri, "uri");
			FileName = ArgumentValidator.AssertNotNull(isolatedStoragefileName, "isolatedStoragefileName");;
			IsolateStorageDirectory = ArgumentValidator.AssertNotNull(isolateStorageDirectory, "isolateStorageDirectory");;
		}
	
		public void Download()
		{
			webClient = new WebClient();
			webClient.DownloadStringCompleted += webClient_DownloadStringCompleted;
			webClient.DownloadStringAsync(Uri);  
		}

		void webClient_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
		{
			webClient.DownloadStringCompleted -= webClient_DownloadStringCompleted;
			if (e.Error != null)
			{
				Error = e.Error;
				OnDownloadComplete(e);
				return;
			}
			string pageContent = e.Result;

			string newUrl = "http://www.calciumsdk.net/CalciumSdkVideos/CalciumGettingStarted.wmv";
			webClient.OpenReadCompleted += OnWebClientOnOpenReadCompleted;
			webClient.OpenReadAsync(new Uri(newUrl, UriKind.Absolute));
		}

		void OnWebClientOnOpenReadCompleted(object sender, OpenReadCompletedEventArgs args)
		{
			webClient.OpenReadCompleted -= OnWebClientOnOpenReadCompleted;
			if (args.Error != null)
			{
				Error = args.Error;
				OnDownloadComplete(args);
				return;
			}

			try
			{
				SaveFile(args.Result);
			}
			catch (Exception ex)
			{
				Error = ex;
			}
			finally
			{
				OnDownloadComplete(args);
			}
		}

		void SaveFile(Stream stream)
		{
			using (IsolatedStorageFile isolatedStorageFile
						= IsolatedStorageFile.GetUserStoreForApplication())
			{
				if (!isolatedStorageFile.DirectoryExists(IsolateStorageDirectory))
				{
					isolatedStorageFile.CreateDirectory(IsolateStorageDirectory);
				}

				using (IsolatedStorageFileStream writeStream
					= new IsolatedStorageFileStream(
							FileName,
							FileMode.Create, isolatedStorageFile))
				{
					CopyStreamBytes(stream, writeStream);
				}
			}
		}

		void CopyStreamBytes(
			Stream fromStream, Stream toStream, bool closeToStream = true)
		{
			if (toStream.CanWrite)
			{
				byte[] fileBytes = ReadStreamBytes(fromStream);
				toStream.Write(fileBytes, 0, fileBytes.Length);
				if (closeToStream)
				{
					toStream.Close();
				}
			}
		}
		
		byte[] ReadStreamBytes(Stream fileStream)
		{
			/* Read the source file into a byte array. */
			byte[] bytes = new byte[fileStream.Length];
			int readLength = (int)fileStream.Length;
			int bytesRead = 0;
			while (readLength > 0)
			{
				/* Read may return anything from 0 to readLength. */
				int read = fileStream.Read(bytes, bytesRead, readLength);

				/* When no bytes left to read it is the end of the file. */
				if (read == 0)
				{
					break;
				}

				bytesRead += read;
				readLength -= read;
			}
			return bytes;
		}

		#region event DownloadComplete

		public event EventHandler<EventArgs> DownloadComplete;

		protected void OnDownloadComplete(EventArgs e)
		{
			var eventCopy = DownloadComplete;
			if (eventCopy != null)
			{
				eventCopy(this, e);
			}
		}

		#endregion
	}
}
