﻿using System;
using System.Windows.Input;
using System.Windows.Media;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class MediaViewModel : ViewModelBase
	{
		public MediaViewModel() : base("Media View")
		{
			playCommand = new DelegateCommand(
				obj =>
				{
						PlayerState = PlayerState.Playing;
						CanPlay = false;
						CanPause = true;
				});

			pauseCommand = new DelegateCommand(
				obj =>
				{
					PlayerState = PlayerState.Paused;
					CanPause = false;
					CanPlay = true;
				});

			muteCommand = new DelegateCommand(obj => Muted = true);

			unMuteCommand = new DelegateCommand(obj => Muted = false);

//			PropertyChanged += delegate { UpdateCommands(); };
		}

//		void UpdateCommands()
//		{
//			pauseCommand.RaiseCanExecuteChanged();
//			playCommand.RaiseCanExecuteChanged();
//			muteCommand.RaiseCanExecuteChanged();
//			unMuteCommand.RaiseCanExecuteChanged();
//		}

		PlayerState playerState;

		public PlayerState PlayerState
		{
			get
			{
				return playerState;
			}
			private set
			{
				Assign(() => PlayerState, ref playerState, value);
			}
		}


		Uri mediaUri = new Uri("http://www.calciumsdk.net/CalciumSdkVideos/CalciumGettingStarted.wmv", UriKind.Absolute);
			/* To test an audio file use the following: */
			// http://files.sparklingclient.com/099_2010.07.09_WP7_Phones_In_The_Wild.mp3
			

		public Uri MediaUri
		{
			get
			{
				return mediaUri;
			}
			set
			{
				Assign(() => MediaUri, ref mediaUri, value);
			}
		}


		double progress;

		public double Progress
		{
			get
			{
				return progress;
			}
			private set
			{
				Assign(() => Progress, ref progress, value);
			}
		}	

		double bufferingProgress;

		public double BufferingProgress
		{
			get
			{
				return bufferingProgress;
			}
			set
			{
				Assign(() => BufferingProgress, ref bufferingProgress, value);
			}
		}

		double downloadProgress;

		public double DownloadProgress
		{
			get
			{
				return downloadProgress;
			}
			set
			{
				Assign(() => DownloadProgress, ref downloadProgress, value);
			}
		}

		double downloadProgressOffset;

		public double DownloadProgressOffset
		{
			get
			{
				return downloadProgressOffset;
			}
			set
			{
				Assign(() => DownloadProgressOffset, ref downloadProgressOffset, value);
			}
		}

		TimeSpan position;

		public TimeSpan Position
		{
			get
			{
				return position;
			}
			set
			{
				position = value;
				/* We cannot place a two way databinding, 
				 * with the MediaPlayer Position property, 
				 * to a property that raises the PropertyChanged event 
				 * for that property. Doing so will cause the MediaElement 
				 * to attempt to continually update itself. 
				 * This is why we use to properties: one for the slider 
				 * and one for the MediaElement. */
				OnPropertyChanged("SliderPosition");
			}
		}

		public TimeSpan SliderPosition
		{
			get
			{
				return position;
			}
			set
			{
				Assign(() => Position, ref position, value);
			}
		}

		bool canSeek;

		public bool CanSeek
		{
			get
			{
				return canSeek;
			}
			set
			{
				Assign(() => CanSeek, ref canSeek, value);
			}
		}

		bool canPause;

		public bool CanPause
		{
			get
			{
				return canPause;
			}
			set
			{
				Assign(() => CanPause, ref canPause, value);
				//UpdateCommands();
			}
		}

		bool canPlay = true;

		public bool CanPlay
		{
			get
			{
				return canPlay;
			}
			set
			{
				Assign(() => CanPlay, ref canPlay, value);
			}
		}	

		TimeSpan bufferingTime = TimeSpan.FromSeconds(10);

		public TimeSpan BufferingTime
		{
			get
			{
				return bufferingTime;
			}
			set
			{
				Assign(() => BufferingTime, ref bufferingTime, value);
			}
		}

		bool muted;

		public bool Muted
		{
			get
			{
				return muted;
			}
			private set
			{
				Assign(() => Muted, ref muted, value);
			}
		}

		double volume = 1;

		public double Volume
		{
			get
			{
				return volume;
			}
			set
			{
				Assign(() => Volume, ref volume, value);
			}
		}

		TimeSpan playLength;

		public TimeSpan PlayLength
		{
			get
			{
				return playLength;
			}
			set
			{
				Assign(() => PlayLength, ref playLength, value);
			}
		}

		bool autoPlay;

		public bool AutoPlay
		{
			get
			{
				return autoPlay;
			}
			private set
			{
				Assign(() => AutoPlay, ref autoPlay, value);
			}
		}	

		MediaElementState mediaElementState;

		public MediaElementState MediaElementState
		{
			get
			{
				return mediaElementState;
			}
			set
			{
				Assign(() => MediaElementState, ref mediaElementState, value);
				//UpdateCommands();
			}
		}

		bool paused;

		public bool Paused
		{
			get
			{
				return paused;
			}
			set
			{
				Assign(() => Paused, ref paused, value);
			}
		}

		double balance;

		public double Balance
		{
			get
			{
				return balance;
			}
			set
			{
				Assign(() => Balance, ref balance, value);
			}
		}
	
		readonly DelegateCommand playCommand;

		public ICommand PlayCommand
		{
			get
			{
				return playCommand;
			}
		}

		readonly DelegateCommand pauseCommand;

		public ICommand PauseCommand
		{
			get
			{
				return pauseCommand;
			}
		}

		readonly DelegateCommand muteCommand;

		public ICommand MuteCommand
		{
			get
			{
				return muteCommand;
			}
		}

		readonly DelegateCommand unMuteCommand;

		public ICommand UnMuteCommand
		{
			get
			{
				return unMuteCommand;
			}
		}

		public void HandleMediaFailed(Exception exception)
		{
			Message = "Problem loading media: " + exception.Message;
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
				Assign(() => Message, ref message, value);
			}
		}
	}
}
