﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

using DanielVaughan.Collections;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class BulkListViewModel : ViewModelBase
	{
		public BulkListViewModel()
		{
			performBulkUpdateCommand = new DelegateCommand<ListUpdateMode>(
				mode =>
					{
						int startIndex = items.Count;
						const int listSize = 10000;
						var list = Enumerable.Range(startIndex, listSize)
										.Select(x => x.ToString()).ToList();

						DateTime startTime = DateTime.Now;
						
						if (mode == ListUpdateMode.OneByOne)
						{
							foreach (var item in list)
							{
								items.Add(item);
							}
						}
						else
						{
							Items.AddRange(list);
						}
						
						TimeSpan timeTaken = DateTime.Now - startTime;
						string message = string.Format("Time to update ({0}): {1}", 
														mode, timeTaken);
						if (mode == ListUpdateMode.OneByOne)
						{
							Message0 = message;
						}
						else
						{
							Message1 = message;
						}
					});
		}

		IList<string> items = new ObservableList<string>();
		// IList<string> items = new ObservableCollection<string>();

		public IList<string> Items
		{
			get
			{
				return items;
			}
			private set
			{
				Assign(() => Items, ref items, value);
			}
		}

		readonly DelegateCommand<ListUpdateMode> performBulkUpdateCommand;

		public DelegateCommand<ListUpdateMode> PerformBulkUpdateCommand
		{
			get
			{
				return performBulkUpdateCommand;
			}
		}

		string message0;

		public string Message0
		{
			get
			{
				return message0;
			}
			private set
			{
				Assign(() => Message0, ref message0, value);
			}
		}

		string message1;

		public string Message1
		{
			get
			{
				return message1;
			}
			private set
			{
				Assign(() => Message1, ref message1, value);
			}
		}
	}

	public enum ListUpdateMode
	{
		OneByOne,
		Range
	}

//	public class BulkDataItem
//	{
//		public string Value { get; set; }
//
//		public BulkDataItem(string value)
//		{
//			Value = value;
//		}
//	}
}
