﻿using System;
using System.Collections.Generic;
using System.Linq;

using DanielVaughan.Services;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class TwitterTimelineViewModel : ViewModelBase
	{
		readonly ITwitterService twitterService;

		public TwitterTimelineViewModel(ITwitterService twitterService)
		{
			this.twitterService = ArgumentValidator.AssertNotNull(twitterService, "twitterService");

			loadUserTimelineCommand = new DelegateCommand<string>(LoadTimeline);
		}

		void LoadTimeline(string twitterScreenName)
		{
			if (string.IsNullOrWhiteSpace(twitterScreenName))
			{
				var messageService = Dependency.Resolve<IMessageService>();
				messageService.ShowError("Please provide a twitter screen name.");

				var navigationService = Dependency.Resolve<INavigationService>();
				navigationService.GoBack();

				return;
			}

			if (twitterScreenName == screenName)
			{
				return;
			}

			Busy = true;

			try
			{
				Message = "Retrieving cached timeline from database.";

				twitterService.GetCachedTimeline(twitterScreenName,
					args =>
					{
						HandleGetTimelineResult(args);
						Message = "Retrieving real data from Twitter.";
						Busy = true;
						twitterService.GetTimeline(
							twitterScreenName, HandleGetTimelineResult);
					});
			}
			catch (Exception ex)
			{
				Busy = false;
				Message = string.Format("Error retrieving items.");
				Console.WriteLine(ex);
			}
		}

		void HandleGetTimelineResult(ResultEventArgs<TwitterUser> e)
		{
			Busy = false;

			if (e.Error != null)
			{
				Message = "Unable to retrieve timeline.";
				return;
			}

			if (e.Result == null)
			{
				Message = "No result";
				return;
			}

			Message = "Received result.";
			TimelineItems = e.Result.TimelineItems.ToList();
			ScreenName = e.Result.ScreenName;
			ImageUrl = e.Result.ImageUrl;
		}

		IEnumerable<TimelineItem> timelineItems;

		public IEnumerable<TimelineItem> TimelineItems
		{
			get
			{
				return timelineItems;
			}
			private set
			{
				Assign(() => TimelineItems, ref timelineItems, value);
			}
		}

		string screenName;

		public string ScreenName
		{
			get
			{
				return screenName;
			}
			private set
			{
				Assign(() => ScreenName, ref screenName, value);
			}
		}

		string imageUrl;

		public string ImageUrl
		{
			get
			{
				return imageUrl;
			}
			private set
			{
				Assign(() => ImageUrl, ref imageUrl, value);
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			private set
			{
				Assign(() => Message, ref message, value);
			}
		}

		bool busy;

		public bool Busy
		{
			get
			{
				return busy;
			}
			private set
			{
				Assign(() => Busy, ref busy, value);
			}
		}

		readonly DelegateCommand<string> loadUserTimelineCommand;

		public DelegateCommand<string> LoadUserTimelineCommand
		{
			get
			{
				return loadUserTimelineCommand;
			}
		}
	}
}
