﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Xml.Linq;

using DanielVaughan.Collections;
using DanielVaughan.Data;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class VersionTolerantViewModel : ViewModelBase, IXmlConvertible
	{
		public VersionTolerantViewModel()
		{
			/* Intentionally left blank. */
		}

		ObservableCollection<User> users = new ObservableCollection<User>();

		public ObservableCollection<User> Users
		{
			get
			{
				return users;
			}
			private set
			{
				users = value;
			}
		}

		const string rootElementName = "VersionTolerantViewModel";

		public void FromXElement(XElement element)
		{
			ArgumentValidator.AssertNotNull(element, "element");
			var rootElement = element.Element(rootElementName) ?? element;

			var version = (int)rootElement.Element("Version");

			var usersElement = rootElement.Element("Users");
			if (usersElement == null)
			{
				throw new ArgumentException("Does not contain Users element.");
			}

			foreach (var browserElement in usersElement.Elements())
			{
				User viewModel = new User(browserElement);
				Users.Add(viewModel);
			}
		}

		public XElement ToXElement()
		{
			XElement element = new XElement(rootElementName,
								new XElement("Version", 1),
								new XElement("Users",
									from user in users
									select user.ToXElement()));

			return element;
		}
	}

	public class User : IXmlConvertible
	{
		public string Name { get; private set; }
		public string Url { get; private set; }
		const string rootElementName = "User";

		public User()
		{
			/* Intentionally left blank. */
		}

		public User(XElement element)
		{
			FromXElement(element);
		}

		public void FromXElement(XElement element)
		{
			ArgumentValidator.AssertNotNull(element, "element");
			var rootElement = element.Element(rootElementName);
			if (rootElement == null)
			{
				throw new ArgumentException(
					string.Format("Expected {0} element.", rootElementName));
			}

			var version = (int)rootElement.Element("Version");

			Name = rootElement.Element("Name").Value;

			if (version < 2)
			{
				return;
			}

			Url = rootElement.Element("Url").Value;
		}

		public XElement ToXElement()
		{
			XElement element = new XElement(rootElementName,
									new XElement("Version", 2),
									new XElement("Name", Name),
									new XElement("Url", Url));

			return element;
		}
	}
}
