﻿using System;
using System.Collections.Generic;
using System.Threading;

using Microsoft.Phone.Info;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
    public class DeviceStatusViewModel : ViewModelBase
    {
        double bytesPerMegabyte = 1048576;
        Timer timer;

        public DeviceStatusViewModel() : base("device status")
        {
            /* Intentionally left blank. */
        }

        public void Start()
        {
            DeviceStatus.KeyboardDeployedChanged += HandleKeyboardDeployedChanged;
            DeviceStatus.PowerSourceChanged += HandlePowerSourceChanged;
            timer = new Timer(delegate { OnPropertyChanged(string.Empty);}, null, 2, 2);
        }

        void HandlePowerSourceChanged(object sender, EventArgs e)
        {
            PowerSource powerSource = DeviceStatus.PowerSource;
            MessageService.ShowMessage("Power source changed to " 
                                            + DeviceStatus.PowerSource);
        }

        void HandleKeyboardDeployedChanged(object sender, EventArgs e)
        {
            bool keyboardDeployed = DeviceStatus.IsKeyboardDeployed;
            MessageService.ShowMessage("Keyboard deployment changed to "
                                            + keyboardDeployed);
        }

        public void Stop()
        {
            timer.Dispose();
            DeviceStatus.KeyboardDeployedChanged -= HandleKeyboardDeployedChanged;
            DeviceStatus.PowerSourceChanged -= HandlePowerSourceChanged;
        }

        public double ApplicationCurrentMemoryUsage
        {
            get
            {
                return DeviceStatus.ApplicationCurrentMemoryUsage / bytesPerMegabyte;
            }
        }

        public double ApplicationMemoryUsageLimit
        {
            get
            {
                return DeviceStatus.ApplicationMemoryUsageLimit / bytesPerMegabyte;
            }
        }

        public double ApplicationPeakMemoryUsage
        {
            get
            {
                return DeviceStatus.ApplicationPeakMemoryUsage / bytesPerMegabyte;
            }
        }

        public string DeviceFirmwareVersion
        {
            get
            {
                return DeviceStatus.DeviceFirmwareVersion;
            }
        }

        public string DeviceHardwareVersion
        {
            get
            {
                return DeviceStatus.DeviceHardwareVersion;
            }
        }

        public string DeviceManufacturer
        {
            get
            {
                return DeviceStatus.DeviceManufacturer;
            }
        }

        public string DeviceName
        {
            get
            {
                return DeviceStatus.DeviceName;
            }
        }

        public double DeviceTotalMemory
        {
            get
            {
                return DeviceStatus.DeviceTotalMemory / bytesPerMegabyte;
            }
        }

        public bool IsKeyboardDeployed
        {
            get
            {
                return DeviceStatus.IsKeyboardDeployed;
            }
        }

        public bool IsKeyboardPresent
        {
            get
            {
                return DeviceStatus.IsKeyboardPresent;
            }
        }

        public PowerSource PowerSource
        {
            get
            {
                return DeviceStatus.PowerSource;
            }
        }
    }
}
