﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Windows.Input;

using DanielVaughan.WindowsPhone7Unleashed.InputValidation;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class AsyncValidationViewModel : ViewModelBase
	{
		public AsyncValidationViewModel()
		{
			AddValidationProperty(() => ValidatedString1);
			AddValidationProperty(() => ValidatedString2);

			submitCommand = new DelegateCommand(
				delegate
			    {
					IsComplete(
						() => Message = "Data Submitted!", 
						() => Message = string.Empty,
						obj => Message = "An error occured: " + obj.Message);
			    });
		}

		public override void BeginValidation(string memberName, object value)
		{
			try
			{
				/* Perform validation asynchronously. */
				ThreadPool.QueueUserWorkItem(state => Validate(memberName, value));
			}
			catch (Exception ex)
			{
				OnValidationComplete(
					new ValidationCompleteEventArgs(memberName, ex));
			}
		}

		void Validate(string propertyName, object value)
		{
		    try
		    {
		        IEnumerable<DataValidationError> errors 
		            = GetPropertyErrors(propertyName, value);
		        if (errors != null && errors.Count() > 0)
		        {
		            OnValidationComplete(
		                new ValidationCompleteEventArgs(propertyName, errors));
		        }
		        else
		        {
		            OnValidationComplete(
		                new ValidationCompleteEventArgs(propertyName));
		        }
            }
            catch (Exception ex)
            {
                OnValidationComplete(
                    new ValidationCompleteEventArgs(propertyName, ex));
            }
		}

		protected override IEnumerable<DataValidationError> GetPropertyErrors(
			string propertyName, object value)
		{
			if (!string.IsNullOrWhiteSpace((string)value))
			{
				yield break;
			}

			switch (propertyName)
			{
				case "ValidatedString1":
					yield return new DataValidationError(1,
						"ValidatedString1 is required.");
					break;
				case "ValidatedString2":
					yield return new DataValidationError(2,
						"ValidatedString2 is required.");
					break;
			}
			yield break;
		}

		#region SubmitCommand property and field

		readonly DelegateCommand submitCommand;

		public ICommand SubmitCommand
		{
			get
			{
				return submitCommand;
			}
		}

		#endregion

		#region Message property and field

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			private set
			{
				Assign("Message", ref message, value);
			}
		}

		#endregion

		#region ValidatedString1 property and field

		string validatedString1;

		public string ValidatedString1
		{
			get
			{
				return validatedString1;
			}
			set
			{
			    Assign(() => ValidatedString1, ref validatedString1, value);
			}
		}

		#endregion

		#region ValidatedString2 property and field

		string validatedString2;

		public string ValidatedString2
		{
			get
			{
				return validatedString2;
			}
			set
			{
			    Assign(() => ValidatedString2, ref validatedString2, value);
			}
		}

		#endregion
	}
}
