﻿using System.Windows.Input;

using Microsoft.Phone.Tasks;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class EmailViewModel : ViewModelBase
	{
		readonly EmailAddressChooserTask emailAddressChooserTask
			= new EmailAddressChooserTask();

		public EmailViewModel() : base("email")
		{
			emailAddressChooserTask.Completed += HandleEmailAddressChooserTaskCompleted;

			chooseRecipientCommand = new DelegateCommand(
				obj => emailAddressChooserTask.Show());

			composeCommand = new DelegateCommand(
				delegate
					{
						EmailComposeTask emailComposeTask 
							= new EmailComposeTask
							  	{
							  		To = toAddress, 
									//Cc = ccAddress, 
									Subject = "Windows Phone 7 Unleashed", 
									Body = "Hi from Windows Phone 7!"
							  	};
						emailComposeTask.Show();
					});
		}

		void HandleEmailAddressChooserTaskCompleted(object sender, EmailResult e)
		{
			if (e.TaskResult == TaskResult.OK)
			{
				ToAddress = e.Email;
			}
			else if (e.Error != null)
			{
				Message = "Unable to choose email address. " + e.ToString();
			}
			else if (e.TaskResult == TaskResult.Cancel)
			{
				Message = "Cancelled";
			}
		}

		string toAddress;

		public string ToAddress
		{
			get
			{
				return toAddress;
			}
			set
			{
				Assign(() => ToAddress, ref toAddress, value);
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
				Assign(() => Message, ref message, value);
			}
		}

		readonly DelegateCommand chooseRecipientCommand;

		public ICommand ChooseRecipientCommand
		{
			get
			{
				return chooseRecipientCommand;
			}
		}

		readonly DelegateCommand composeCommand;

		public ICommand ComposeCommand
		{
			get
			{
				return composeCommand;
			}
		}
	}
}
