﻿using System.Collections.Generic;
using System.Windows.Input;

using Microsoft.Phone.Tasks;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class MarketplaceViewModel : ViewModelBase
	{
		public MarketplaceViewModel()
		{
			detailCommand = new DelegateCommand(
					delegate
					{
						MarketplaceDetailTask task = new MarketplaceDetailTask();
						task.Show();
					});

			hubCommand = new DelegateCommand(
				delegate
					{
						MarketplaceHubTask task = new MarketplaceHubTask();
						if (contentType.HasValue)
						{
							task.ContentType = contentType.Value;
						}
						task.Show();
					});

			reviewCommand = new DelegateCommand(
				delegate
					{
						MarketplaceReviewTask task = new MarketplaceReviewTask();
						task.Show();
					});

			searchCommand = new DelegateCommand(
				delegate
					{
						MarketplaceSearchTask task = new MarketplaceSearchTask
								{
									/* SearchTerms can't be null or empty,
									 * or else an exception will be thrown. */
									SearchTerms = string.IsNullOrEmpty(searchTerms) 
														? " " : searchTerms
								};
						if (contentType.HasValue)
						{
							task.ContentType = contentType.Value;
						}
						task.Show();
					});
		}

		readonly List<MarketplaceContentType> contentTypes 
			= new List<MarketplaceContentType>
			  	{
			  		MarketplaceContentType.Applications, 
					MarketplaceContentType.Music
			  	};

		public IEnumerable<MarketplaceContentType> ContentTypes
		{
			get
			{
				return contentTypes;
			}
		}

		MarketplaceContentType? contentType;

		public MarketplaceContentType? ContentType
		{
			get
			{
				return contentType;
			}
			set
			{
				Assign("ContentType", ref contentType, value);
			}
		}

		readonly DelegateCommand detailCommand;

		public ICommand DetailCommand
		{
			get
			{
				return detailCommand;
			}
		}

		readonly DelegateCommand hubCommand;

		public ICommand HubCommand
		{
			get
			{
				return hubCommand;
			}
		}

		readonly DelegateCommand reviewCommand;

		public ICommand ReviewCommand
		{
			get
			{
				return reviewCommand;
			}
		}

		readonly DelegateCommand searchCommand;

		public ICommand SearchCommand
		{
			get
			{
				return searchCommand;
			}
		}

		string searchTerms = "puzzle";

		public string SearchTerms
		{
			get
			{
				return searchTerms;
			}
			set
			{
				Assign("SearchTerms", ref searchTerms, value);
			}
		}

	}
}
