﻿using System.Windows.Input;

using Microsoft.Phone.Tasks;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class PhoneCallViewModel : ViewModelBase
	{
		readonly PhoneNumberChooserTask phoneNumberChooserTask 
			= new PhoneNumberChooserTask();

		public PhoneCallViewModel() : base("Phone Call Tasks")
		{
			phoneNumberChooserTask.Completed += HandlePhoneNumberChooserTaskCompleted;	

			chooseCommand = new DelegateCommand(
				obj => phoneNumberChooserTask.Show());

			callCommand = new DelegateCommand(
				delegate
					{
						Message = string.Empty;

						PhoneCallTask phoneCallTask = new PhoneCallTask
						                              	{
						                              		DisplayName = "someone", 
															PhoneNumber = phoneNumber
						                              	};
						phoneCallTask.Show();
					});
		}

		void HandlePhoneNumberChooserTaskCompleted(object sender, PhoneNumberResult e)
		{
			if (e.TaskResult == TaskResult.OK)
			{
				PhoneNumber = e.PhoneNumber;
			}
			else if (e.Error != null)
			{
				Message = "Unable to choose number. " + e.ToString();
			}
			else if (e.TaskResult == TaskResult.Cancel)
			{
				Message = "Cancelled";
			}
		}

		string phoneNumber;

		public string PhoneNumber
		{
			get
			{
				return phoneNumber;
			}
			set
			{
				Assign(() => PhoneNumber, ref phoneNumber, value);
			}
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
				Assign(() => Message, ref message, value);
			}
		}

		readonly DelegateCommand chooseCommand;

		public ICommand ChooseCommand
		{
			get
			{
				return chooseCommand;
			}
		}

		readonly DelegateCommand callCommand;

		public ICommand CallCommand
		{
			get
			{
				return callCommand;
			}
		}
	}
}
