﻿using System.Windows.Input;
using System.Windows.Media.Imaging;

using Microsoft.Phone.Tasks;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	public class PhotoChooserViewModel : ViewModelBase
	{
	    readonly PhotoChooserTask task = new PhotoChooserTask();

		public PhotoChooserViewModel()
		{
			task.Completed += HandleTaskCompleted;	
			chooseCommand = new DelegateCommand(
				delegate
					{
						task.PixelWidth = pixelWidth;
						task.PixelHeight = pixelHeight;
						task.ShowCamera = showCamera;
						task.Show();
					});
		}

		void HandleTaskCompleted(object sender, PhotoResult e)
		{
			if (e.TaskResult == TaskResult.OK)
			{
				BitmapImage bitmapImage = new BitmapImage();
				bitmapImage.SetSource(e.ChosenPhoto);
				CapturedImage = bitmapImage;
				PixelWidth = ImageWidth = bitmapImage.PixelWidth;
				PixelHeight = ImageHeight = bitmapImage.PixelHeight;
				OriginalFileName = e.OriginalFileName;
			}

            if (e.Error != null)
            {
                MessageService.ShowError("An error occurred. " + e.Error.ToString());
            }
		}

		readonly DelegateCommand chooseCommand;

		public ICommand ChooseCommand
		{
			get
			{
				return chooseCommand;
			}
		}

		BitmapImage capturedImage;

		public BitmapImage CapturedImage
		{
			get
			{
				return capturedImage;
			}
			set
			{
				Assign(() => CapturedImage, ref capturedImage, value);
			}
		}

		string originalFileName;

		public string OriginalFileName
		{
			get
			{
				return originalFileName;
			}
			set
			{
				Assign(() => OriginalFileName, ref originalFileName, value);
			}
		}

		int pixelWidth = 100;

		public int PixelWidth
		{
			get
			{
				return pixelWidth;
			}
			set
			{
				Assign(() => PixelWidth, ref pixelWidth, value);
			}
		}

		int pixelHeight = 100;

		public int PixelHeight
		{
			get
			{
				return pixelHeight;
			}
			set
			{
				Assign(() => PixelHeight, ref pixelHeight, value);
			}
		}

		bool showCamera;

		public bool ShowCamera
		{
			get
			{
				return showCamera;
			}
			set
			{
				Assign(() => ShowCamera, ref showCamera, value);
			}
		}

		int imageWidth;

		public int ImageWidth
		{
			get
			{
				return imageWidth;
			}
			set
			{
				Assign(() => ImageWidth, ref imageWidth, value);
			}
		}

		int imageHeight;

		public int ImageHeight
		{
			get
			{
				return imageHeight;
			}
			set
			{
				Assign(() => ImageHeight, ref imageHeight, value);
			}
		}
	}
}
