﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Threading;

using DanielVaughan.ComponentModel;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class LocalizabilityViewModel : ViewModelBase
	{
		public LocalizabilityViewModel()
		{
			cultureName = supportedCultures[0];
			SetCurrentCultureValues();
		}

		CultureName cultureName;

		public CultureName CultureName
		{
			get
			{
				return cultureName;
			}
			set
			{
				CultureName temp = cultureName;
				AssignmentResult result 
                    = Assign(() => CultureName, ref cultureName, value);

				if (result == AssignmentResult.Success && value != null)
				{
					try
					{
						SetCulture(value.CultureCode);
					}
					catch (PlatformNotSupportedException)
					{
						CultureName = temp;
						Message = "Platform not supported";
						return;
					}
				}
			}
		}

		readonly List<CultureName> supportedCultures = new List<CultureName>
										{
											new CultureName("en-US", "English US"),
											new CultureName("fr-FR", "Français"),
											/* Not supported. */
											new CultureName("ar-AE", "العربية") 
										};

		public IEnumerable<CultureName> SupportedCultures
		{
			get
			{
				return supportedCultures;
			}
		}

		void SetCulture(string cultureName)
		{
			CultureInfo cultureInfo = new CultureInfo(cultureName);

			Thread.CurrentThread.CurrentCulture = cultureInfo;
			Thread.CurrentThread.CurrentUICulture = cultureInfo;

			BindableResources.RaiseNotifyPropertyChanged();
			Message = string.Empty;

			SetCurrentCultureValues();
		}

		void SetCurrentCultureValues()
		{
			CultureInfo culture = Thread.CurrentThread.CurrentCulture;
			CultureInfo uiCulture = Thread.CurrentThread.CurrentUICulture;
			DisplayLanguage = uiCulture.DisplayName;
			RegionalFormat = culture.NativeName;
			DateTime date = DateTime.Now;
			try
			{
				DateFormatLong = culture.DateTimeFormat.LongDatePattern 
					                        + " " + date.ToString("D");
				DateFormatShort = culture.DateTimeFormat.ShortDatePattern 
					                        + " " + date.ToString("d");
				TimeFormat = culture.DateTimeFormat.LongTimePattern + " " 
					                        + date.ToString("T");
			}
			catch (IndexOutOfRangeException)
			{
				Message = "DateTimeFormat is not defined for culture.";
			}
			const int monetaryAmount = 123456789;
			CurrencyFormat = monetaryAmount.ToString("C");
		}

		string message;

		public string Message
		{
			get
			{
				return message;
			}
			set
			{
			    Assign(() => Message, ref message, value);
			}
		}

		string displayLanguage;

		public string DisplayLanguage
		{
			get
			{
				return displayLanguage;
			}
			set
			{
			    Assign(() => DisplayLanguage, ref displayLanguage, value);
			}
		}

		string regionalFormat;

		public string RegionalFormat
		{
			get
			{
				return regionalFormat;
			}
			set
			{
			    Assign(() => RegionalFormat, ref regionalFormat, value);
			}
		}

		string dateFormatLong;

		public string DateFormatLong
		{
			get
			{
				return dateFormatLong;
			}
			set
			{
			    Assign(() => DateFormatLong, ref dateFormatLong, value);
			}
		}

		string dateFormatShort;

		public string DateFormatShort
		{
			get
			{
				return dateFormatShort;
			}
			set
			{
			    Assign(() => DateFormatShort, ref dateFormatShort, value);
			}
		}

		string timeFormat;

		public string TimeFormat
		{
			get
			{
				return timeFormat;
			}
			set
			{
			    Assign(() => TimeFormat, ref timeFormat, value);
			}
		}

		string currencyFormat;

		public string CurrencyFormat
		{
			get
			{
				return currencyFormat;
			}
			set
			{
			    Assign(() => CurrencyFormat, ref currencyFormat, value);
			}
		}
	}
}
