﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Windows.Input;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class MainPageViewModel : ViewModelBase
	{
		class Categories
		{
			public const string BingMaps = "Bing Maps";
			public const string UnitTesting = "Unit Testing";
			public const string Controls = "Controls";
			public const string SilverlightToolkit = "Silverlight Toolkit";
		}

		List<CustomGrouping<LinkInfo>> links;// = new List<CustomGrouping<LinkInfo>>();

		public MainPageViewModel()
		{
            /* These pages should not be directly navigable. */
            List<string> excludedPages = new List<string>
                                             {
                                                 "customdatepickerview.xaml",
												 "twittertimelineview.xaml"
                                             };
		    var sortedLinks = ProjectXamlMeta.Links.Where(
									x => !excludedPages.Contains(x.Title.ToLower())).OrderBy(info => info.Uri.ToString());
            links = (from linkInfo in sortedLinks
					 group linkInfo by linkInfo.Category.Contains("/") 
						? linkInfo.Category.Substring(0, linkInfo.Category.IndexOf("/")) 
						: linkInfo.Category into grouping
				     select new CustomGrouping<LinkInfo>(
					       grouping.Key, grouping.AsEnumerable())).ToList();

			int count = 0;
			foreach (var link in links)
			{
				count += link.Count();
			}

			const string navigationKey = "HomepageNavigationUrl";

			saveUrlCommand = new DelegateCommand<string>(
				arg =>
					{
						if (string.Compare(arg, "app://external/", 
								StringComparison.CurrentCultureIgnoreCase) == 0)
						{
							return;
						}
						IsolatedStorageSettings.ApplicationSettings[navigationKey] = arg;
						IsolatedStorageSettings.ApplicationSettings.Save();
					});

			refreshCommand = new DelegateCommand(
				obj =>
					{
						try
						{
							string url;
							if (IsolatedStorageSettings.ApplicationSettings.TryGetValue(navigationKey, out url))
							{
								LastUri = new Uri(url, UriKind.Relative);
								int indexOfSlash = url.LastIndexOf("/");
								LastUriTitle = indexOfSlash > 0 ? url.Substring(url.LastIndexOf("/") + 1) : url;
							}
						}
						catch (Exception ex)
						{
							/* Ignore */
							Console.WriteLine(ex);
						}
					});
		}

		readonly DelegateCommand navigateToPreviousCommand;

		public ICommand NavigateToPreviousCommand
		{
			get
			{
				return navigateToPreviousCommand;
			}
		}

		readonly DelegateCommand<string> saveUrlCommand;

		public DelegateCommand<string> SaveUrlCommand
		{
			get
			{
				return saveUrlCommand;
			}
		}

		public IEnumerable<CustomGrouping<LinkInfo>> Links
		{
			get
			{
				return links;
			}
		}

		Uri lastUri;

		public Uri LastUri
		{
			get
			{
				return lastUri;
			}
			private set
			{
				Assign(() => LastUri, ref lastUri, value);
			}
		}

		public bool CanNavigateToLastUri
		{
			get
			{
				return lastUri != null;
			}
		}

		string lastUriTitle;

		public string LastUriTitle
		{
			get
			{
				return lastUriTitle;
			}
			private set
			{
				Assign(() => LastUriTitle, ref lastUriTitle, value);
			}
		}

		readonly DelegateCommand refreshCommand;

		public ICommand RefreshCommand
		{
			get
			{
				return refreshCommand;
			}
		}

	}

	public class LinkInfo
	{
		public Uri Uri { get; set; }
		public string Category { get; set; }
		public string Title { get; set; }
		public string Description { get; set; }

		public LinkInfo(Uri uri, string category, string title, string description = null)
		{
			Uri = ArgumentValidator.AssertNotNull(uri, "uri");
			Category = ArgumentValidator.AssertNotNull(category, "category");
			Title = ArgumentValidator.AssertNotNull(title, "title");
			Description = description;
		}

		public LinkInfo(string relativeUrl, string category, string title, string description = null)
			: this(new Uri(relativeUrl, UriKind.Relative), category, title, description)
		{
			/* Intentionally left blank. */
		}
	}
}
