﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;

using DanielVaughan.WindowsPhone7Unleashed.Examples.BookshopServiceReference;

using Microsoft.Phone.Shell;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Navigation
{
	public class ProductDetailsViewModel : ViewModelBase
	{
		const string transientStateKey = "ProductDetailsViewModel_Product";
		readonly IDictionary<string, object> transientStateDictionary;

		internal ProductDetailsViewModel()
		{
			/* Intentionally left blank. */
		}

		public ProductDetailsViewModel(
			IDictionary<string, object> transientStateDictionary)
		{
			this.transientStateDictionary = ArgumentValidator.AssertNotNull(
				transientStateDictionary, "transientStateDictionary");
		}

		public void LoadProduct(int productId)
		{
			object transientState;
			if (PhoneApplicationService.Current.State.TryGetValue(
					transientStateKey, out transientState))
			{
				Product = transientState as Product;
				if (product != null && product.Id == productId)
				{
					return;
				}
			}

			BookshopServiceClient client = new BookshopServiceClient();
			client.GetProductByIdCompleted += (sender, args) =>
			{
				if (args.Error != null)
				{
					throw args.Error;
				}
				Product = args.Result;
			};
			client.GetProductByIdAsync(productId);
		}

		#region Product property and field

		Product product;

		public Product Product
		{
			get
			{
				return product;
			}
			/* Setter is not private to enable sample data. 
			 * See ProductDetailsViewSampleData.xaml */
			internal set
			{
				product = value;
				OnPropertyChanged("Product");
			}
		}

		#endregion

		public void SaveTransientState()
		{
			Debug.WriteLine("ProductDetailsViewModel saving transient state.");
			transientStateDictionary[transientStateKey] = product;
		}
	}
}
