﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Threading;

using DanielVaughan.WindowsPhone7Unleashed.Examples.BookshopServiceReference;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Navigation
{
	public class ProductsViewModel : ViewModelBase
	{
		readonly IDictionary<string, object> transientStateDictionary;
		const string transientStateKey = "ProductsViewModel_Products";

		public ProductsViewModel(
			IDictionary<string, object> transientStateDictionary)
		{
			this.transientStateDictionary = ArgumentValidator.AssertNotNull(
				transientStateDictionary, "transientStateDictionary");

			LoadTransientState();
			if (products != null)
			{
				return;
			}

			BookshopServiceClient client = new BookshopServiceClient();
			client.GetProductsCompleted += (sender, args) =>
			                               	{
												if (args.Error != null)
												{
													MessageService.ShowError("Unable to retrieve products.");
												    return;
												}
												
			                               		Products = args.Result;

												/* Simulate a delay. */ 
												ThreadPool.QueueUserWorkItem(
													delegate
													{
														Wait(3000);
														Loaded = true;
													});
			                               		
												//Loaded = true;
			                               	};
			client.GetProductsAsync();
		}

		ObservableCollection<Product> products;

		public ObservableCollection<Product> Products
		{
			get
			{
				return products;
			}
			private set
			{
				Assign(() => Products, ref products, value);
			}
		}

		bool loaded;

		public bool Loaded
		{
			get
			{
				return loaded;
			}
			private set
			{
				Assign(() => Loaded, ref loaded, value);
				Debug.WriteLine("******loaded = " + loaded);
			}
		}
        
		public void SaveTransientState()
		{
			Debug.WriteLine("ProductsViewModel saving transient state.");
			transientStateDictionary[transientStateKey] = products;
		}

		public void LoadTransientState()
		{
			Debug.WriteLine("ProductsViewModel loading transient state.");
			object transientState;
			if (transientStateDictionary.TryGetValue(
					transientStateKey, out transientState))
			{
				products = transientState as ObservableCollection<Product>;
				if (products != null)
				{
					Loaded = true;
				}
			}
		}

		static readonly object waitLock = new object();

		static void Wait(int delayMs)
		{
			if (delayMs > 0)
			{
				lock (waitLock)
				{
					Monitor.Wait(waitLock, delayMs);
				}
			}
		}
	}
}
