﻿using System;
using System.Diagnostics;
using System.Windows.Input;

using DanielVaughan.Devices.Sensors;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class AccelerometerViewModel : ViewModelBase
	{
		public AccelerometerViewModel(IAccelerometer accelerometer) : base("accelerometer")
		{
			startCommand = new DelegateCommand(
				obj =>
					{
						accelerometer.ReadingChanged -= HandleReadingChanged;
						accelerometer.ReadingChanged += HandleReadingChanged;
						accelerometer.Shake -= HandleShake;
						accelerometer.Shake += HandleShake;
						accelerometer.Start();
					});

			stopCommand = new DelegateCommand(
				obj =>
					{
						accelerometer.ReadingChanged -= HandleReadingChanged;
						accelerometer.Shake -= HandleShake;
						accelerometer.Stop();
					});

			calibrateCommand = new DelegateCommand(
				obj => MessageService.ShowMessage(accelerometer.Calibrate()
				                                  	? "Successfully calibrated."
				                                  	: "Unable to calibrate."), 
				obj => accelerometer.CanCalibrate());
		}

		void HandleShake(object sender, EventArgs e)
		{
			ShakeCount++;
		}

		void HandleReadingChanged(object sender, EventArgs e)
		{
			IAccelerometer accelerometer = (IAccelerometer)sender;
			Reading = accelerometer.Reading;
			UpdateCommands();
		}

		void UpdateCommands()
		{
			calibrateCommand.RaiseCanExecuteChanged();
		}

		readonly DelegateCommand startCommand;

		public ICommand StartCommand
		{
			get
			{
				return startCommand;
			}
		}

		readonly DelegateCommand stopCommand;

		public ICommand StopCommand
		{
			get
			{
				return stopCommand;
			}
		}

		EnhancedAccelerometerReading reading;

		public EnhancedAccelerometerReading Reading
		{
			get
			{
				return reading;
			}
			private set
			{
				Assign("Reading", ref reading, value);
			}
		}

		readonly DelegateCommand calibrateCommand;

		public DelegateCommand CalibrateCommand
		{
			get
			{
				return calibrateCommand;
			}
		}

		int shakeCount;

		public int ShakeCount
		{
			get
			{
				return shakeCount;
			}
			private set
			{
				Assign(() => ShakeCount, ref shakeCount, value);
			}
		}
	}
}
