﻿using System;
using System.Collections.ObjectModel;
using System.IO.IsolatedStorage;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class ThumbnailsViewModel : ViewModelBase
	{
		public ThumbnailsViewModel() : base("photos")
		{
			viewImageCommand = new DelegateCommand<string>(ViewImage);
		}

		void ViewImage(string name)
		{
			Navigate("/Sensors/Camera/MediaLibraryImageView.xaml?image=" + name);
		}

		public void Populate()
		{
			string imageDirectory = PhotoCameraViewModel.ImageDirectory;

			ObservableCollection<ImageInfo> tempImages = new ObservableCollection<ImageInfo>();

			using (IsolatedStorageFile storageFile
							= IsolatedStorageFile.GetUserStoreForApplication())
			{
				if (!storageFile.DirectoryExists(imageDirectory))
				{
					return;
				}

				string[] fileNames = storageFile.GetFileNames(imageDirectory + @"\*.jpg");
				foreach (string fileName in fileNames)
				{
					ImageInfo info = new ImageInfo(
						new Uri(imageDirectory + "/" + fileName, UriKind.Relative), 
						fileName);
					tempImages.Add(info);
				}
			}

			Images = tempImages;
		}

		ObservableCollection<ImageInfo> images = new ObservableCollection<ImageInfo>();

		public ObservableCollection<ImageInfo> Images
		{
			get
			{
				return images;
			}
			private set
			{
				Assign(() => Images, ref images, value);
			}
		}

		readonly DelegateCommand<string> viewImageCommand;

		public DelegateCommand<string> ViewImageCommand
		{
			get
			{
				return viewImageCommand;
			}
		}
	}

	public class ImageInfo
	{
		public string MediaLibraryUrl { get; private set; }
		public Uri ThumbnailUri { get; private set; }

		public ImageInfo(Uri thumbnailUri, string mediaLibraryUrl)
		{
			ThumbnailUri = ArgumentValidator.AssertNotNull(
												thumbnailUri, "thumbnailUri");
			MediaLibraryUrl = ArgumentValidator.AssertNotNullOrEmpty(
												mediaLibraryUrl, "mediaLibraryUrl");
		}
	}
}
