﻿using System;

using DanielVaughan.Devices.Sensors;

using Microsoft.Devices.Sensors;
using Microsoft.Xna.Framework;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class GyroscopeViewModel : ViewModelBase
	{
		Gyroscope gyroscope;
		DateTimeOffset lastUpdateTime = DateTimeOffset.MinValue;
		ThreeDimensionalVector cumulativeRotationRadians;

		public GyroscopeViewModel() : base("gyroscope")
		{
			/* Intentionally left blank. */
		}

		public void Start()
		{
			if (!Gyroscope.IsSupported)
			{
				MessageService.ShowMessage("Gyroscope is not supported on this device.");
				return;
			}

			gyroscope = new Gyroscope();
			// gyroscope.TimeBetweenUpdates = TimeSpan.FromMilliseconds(20); 
			gyroscope.CurrentValueChanged += HandleGyroscopeCurrentValueChanged;
			gyroscope.Start();
		}

		void HandleGyroscopeCurrentValueChanged(
			object sender, SensorReadingEventArgs<GyroscopeReading> e)
		{
			GyroscopeReading reading = e.SensorReading;
			Vector3 rate = reading.RotationRate;
			RotationRate = new ThreeDimensionalVector(rate.X, rate.Y, rate.Z);

			if (lastUpdateTime.Equals(DateTimeOffset.MinValue))
			{
				lastUpdateTime = e.SensorReading.Timestamp;
				cumulativeRotationRadians = rotationRate;
				return;
			}

			TimeSpan timeSinceLastUpdate = e.SensorReading.Timestamp - lastUpdateTime;

			/* Obtain the amount the device rotated since the last update 
			 * by multiplying the rotation rate by the time since the last update. 
			 * (radians/second) * secondsSinceLastReading = radiansSinceLastReading. */
			cumulativeRotationRadians += rotationRate * (timeSinceLastUpdate.TotalSeconds);
			CumulativeRotation = new RotationAngles(
				MathHelper.ToDegrees((float)cumulativeRotationRadians.X), 
				MathHelper.ToDegrees((float)cumulativeRotationRadians.Y), 
				MathHelper.ToDegrees((float)cumulativeRotationRadians.Z));

			lastUpdateTime = e.SensorReading.Timestamp;
		}

		ThreeDimensionalVector rotationRate;

		public ThreeDimensionalVector RotationRate
		{
			get
			{
				return rotationRate;
			}
			private set
			{
				Assign("RotationRate", ref rotationRate, value);
			}
		}

		RotationAngles cumulativeRotation;

		public RotationAngles CumulativeRotation
		{
			get
			{
				return cumulativeRotation;
			}
			private set
			{
				Assign("CumulativeRotation", ref cumulativeRotation, value);
			}
		}

		public void Stop()
		{
			if (gyroscope == null)
			{
				return;
			}

			gyroscope.Stop();
			gyroscope.CurrentValueChanged -= HandleGyroscopeCurrentValueChanged;
			gyroscope.Dispose();
			gyroscope = null;
		}
	}

	public class RotationAngles
	{
		public double X { get; private set; }
		public double Y { get; private set; }
		public double Z { get; private set; }

		public RotationAngles(double x, double y, double z)
		{
			X = x;
			Y = y;
			Z = z;
		}
	}
}
