﻿using System;
using System.Collections.Generic;

using DanielVaughan.Devices.Sensors;

using Microsoft.Devices.Sensors;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.Sensors
{
	public class MotionViewModel : ViewModelBase
	{
		Motion motion;
		DateTimeOffset lastUpdateTime = DateTimeOffset.MinValue;
		ThreeDimensionalVector cumulativeRotationRadians;

		public MotionViewModel() : base("motion")
		{
			/* Intentionally left blank. */
		}

		public void Start()
		{
			if (!Motion.IsSupported)
			{
				MessageService.ShowMessage("Motion is not supported on this device.");
				return;
			}

			motion = new Motion();
			// motion.TimeBetweenUpdates = TimeSpan.FromMilliseconds(20); 
			double interval = motion.TimeBetweenUpdates.TotalMilliseconds;
			motion.CurrentValueChanged += HandleMotionCurrentValueChanged;
			motion.Start();
		}

		void HandleMotionCurrentValueChanged(
			object sender, SensorReadingEventArgs<MotionReading> e)
		{
			MotionReading reading = e.SensorReading;
			AttitudeReading = reading.Attitude;
			DeviceAcceleration = reading.DeviceAcceleration.ToThreeDimensionalVector();
			DeviceRotationRate = reading.DeviceRotationRate.ToThreeDimensionalVector();
			Gravity = reading.Gravity.ToThreeDimensionalVector();
			AttitudeReading attitude = reading.Attitude;
			Attitude = new ThreeDimensionalVector(attitude.Pitch, attitude.Roll, attitude.Yaw);
		}

		public void Stop()
		{
			if (motion == null)
			{
				return;
			}

			motion.Stop();
			motion.CurrentValueChanged -= HandleMotionCurrentValueChanged;
			motion.Dispose();
			motion = null;
		}

		AttitudeReading attitudeReading;

		public AttitudeReading AttitudeReading
		{
			get
			{
				return attitudeReading;
			}
			private set
			{
				Assign("AttitudeReading", ref attitudeReading, value);
			}
		}

		ThreeDimensionalVector attitude;

		public ThreeDimensionalVector Attitude
		{
			get
			{
				return attitude;
			}
			set
			{
				Assign("Attitude", ref attitude, value);
			}
		}

		ThreeDimensionalVector deviceAcceleration = new ThreeDimensionalVector();

		public ThreeDimensionalVector DeviceAcceleration
		{
			get
			{
				return deviceAcceleration;
			}
			private set
			{
				Assign("DeviceAcceleration", ref deviceAcceleration, value);
			}
		}

		ThreeDimensionalVector deviceRotationRate = new ThreeDimensionalVector();

		public ThreeDimensionalVector DeviceRotationRate
		{
			get
			{
				return deviceRotationRate;
			}
			private set
			{
				Assign("DeviceRotationRate", ref deviceRotationRate, value);
			}
		}

		ThreeDimensionalVector gravity = new ThreeDimensionalVector();

		public ThreeDimensionalVector Gravity
		{
			get
			{
				return gravity;
			}
			private set
			{
				Assign("Gravity", ref gravity, value);
			}
		}
	}

}
