﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.ControlExamples
{
	public class ContactListViewModel : ViewModelBase
	{
		const int contactCount = 35;
		
		public ContactListViewModel()
		{
			List<CustomGrouping<Contact>> groupings 
				= (from contact in Contact.GetRandomContacts(contactCount)
					orderby contact.LastName orderby contact.FirstName
					group contact by Char.ToLower(contact.LastName.First()) into grouping
					select new CustomGrouping<Contact>(
						grouping.Key, grouping.AsEnumerable())).ToList();

			/* Populate with groups representing those letters without any items. */
			const string groupCharacters = "abcdefghijklmnopqrstuvwxyz";
			List<Contact> emptyList = new List<Contact>();
			var allGroupings = from character in groupCharacters
							   where !groupings.Any(x => character.Equals(x.Key))
							   select new CustomGrouping<Contact>(character, emptyList);

			contactGroupings = groupings.Union(allGroupings).OrderBy(x => x.Key);
		}

		IEnumerable<CustomGrouping<Contact>> contactGroupings;

		public IEnumerable<CustomGrouping<Contact>> ContactGroupings
		{
			get
			{
				return contactGroupings;
			}
			private set
			{
				Assign(() => ContactGroupings, ref contactGroupings, value);
			}
		}
	}

}
