﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public class CustomGrouping<TElement> : IGrouping<object, TElement>
	{
		readonly IEnumerable<TElement> items;
		readonly object key;

		public CustomGrouping(object key, IEnumerable<TElement> items)
		{
			this.key = ArgumentValidator.AssertNotNull(key, "key");
			this.items = ArgumentValidator.AssertNotNull(items, "items");
		}

		public object Key
		{
			get
			{
				return key;
			}
		}

		public IEnumerator<TElement> GetEnumerator()
		{
			return items.GetEnumerator();
		}

		IEnumerator IEnumerable.GetEnumerator()
		{
			return items.GetEnumerator();
		}

		public bool HasItems
		{
			get
			{
				return items.Any();
			}
		}

		public override bool Equals(object obj)
		{
			var otherGrouping = obj as CustomGrouping<TElement>;
			return otherGrouping != null && Key.Equals(otherGrouping.Key);
		}

		public override int GetHashCode()
		{
			return Key.GetHashCode();
		}
	}
}