﻿using System.Collections.Generic;
using System.Linq;

using DanielVaughan.ComponentModel;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples.ControlExamples
{
	public class AppListViewModel : ViewModelBase
	{
		const int appCount = 30;

		public AppListViewModel()
		{
			GroupByCategory();
		}

		IEnumerable<CustomGrouping<MarketplaceApp>> appGroupings;

		public IEnumerable<CustomGrouping<MarketplaceApp>> AppGroupings
		{
			get
			{
				return appGroupings;
			}
			private set
			{
				Assign(() => AppGroupings, ref appGroupings, value);
			}
		}

		void GroupByCategory()
		{
			AppGroupings = (from app in MarketplaceApp.CreateRandomApps(appCount)
				   group app by app.Category into grouping
				   select new CustomGrouping<MarketplaceApp>(
					   grouping.Key, grouping.AsEnumerable())).ToList();
		}

		void GroupByPrice()
		{
			AppGroupings = (from app in MarketplaceApp.CreateRandomApps(appCount)
				   group app by app.Price into grouping orderby grouping.Key
				   select new CustomGrouping<MarketplaceApp>(
					   grouping.Key, grouping.AsEnumerable())).ToList();
		}

		readonly static IEnumerable<AppGroup> appGroups = new List<AppGroup>
		                                           	{
		                                           		AppGroup.Category, 
														AppGroup.Price
		                                           	};

		public IEnumerable<AppGroup> AppGroups
		{
			get
			{
				return appGroups;
			}
		}

		AppGroup groupBy = appGroups.First();

		public AppGroup GroupBy
		{
			get
			{
				return groupBy;
			}
			set
			{
				if (Assign(() => GroupBy, ref groupBy, value) 
					== AssignmentResult.Success)
				{
					switch (value)
					{
						case AppGroup.Price:
							GroupByPrice();
							break;
						default:
							GroupByCategory();
							break;
					}
				}
				
			}
		}
	}

	public enum AppGroup
	{
		Category,
		Price
	}
}
