﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;

using Microsoft.Phone.Controls;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public partial class GesturesView : PhoneApplicationPage
	{
		readonly SolidColorBrush dragBrush = new SolidColorBrush(Colors.Orange);
		readonly Brush normalBrush;
		readonly SolidColorBrush pinchBrush = new SolidColorBrush(Colors.Red);

		public GesturesView()
		{
			InitializeComponent();

			normalBrush = border.Background;
		}

		void HandleGestureBegin(object sender, GestureEventArgs e)
		{
			Debug.WriteLine("Gesture begin.");
		}

		void HandleGestureCompleted(object sender, GestureEventArgs e)
		{
			Debug.WriteLine("Gesture completed.");
		}

		void HandleTap(object sender, GestureEventArgs e)
		{
			compositeTransform.TranslateX = compositeTransform.TranslateY = 0;
			/* Uncomment the next line to prevent 
			 * the page level handler from being called. */
			//e.Handled = true;
		}

		void HandleDoubleTap(object sender, GestureEventArgs e)
		{
			compositeTransform.ScaleX = compositeTransform.ScaleY = 1;
		}

		void HandleHold(object sender, GestureEventArgs e)
		{
			ResetPosition();
		}

		void ResetPosition()
		{
			compositeTransform.TranslateX = compositeTransform.TranslateY = 0;
			compositeTransform.ScaleX = compositeTransform.ScaleY = 1;
			compositeTransform.Rotation = 0;
		}

		void HandleDragStarted(object sender, DragStartedGestureEventArgs e)
		{
			border.Background = dragBrush;
			Debug.WriteLine("Drag started.");
		}

		void HandleDragDelta(object sender, DragDeltaGestureEventArgs e)
		{
			compositeTransform.TranslateX += e.HorizontalChange;
			compositeTransform.TranslateY += e.VerticalChange;

			messageBlock.Text = string.Format("Drag: Direction {0} \n HorizontalChange {1} \n VerticalChange {2}",
				e.Direction, e.HorizontalChange, e.VerticalChange);
		}

		void HandleDragCompleted(object sender, DragCompletedGestureEventArgs e)
		{
			border.Background = normalBrush;

			string message = string.Format("Drag completed: Direction {0} \n HorizontalChange {1} \n VerticalChange {2} \n HorizontalVelocity {3} \n VerticalVelocity {4}",
				e.Direction, e.HorizontalChange, e.VerticalChange, e.HorizontalVelocity, e.VerticalVelocity);
			messageBlock.Text = message;
			Debug.WriteLine(message);
		}

		double initialAngle;
		double initialScale;

		void HandlePinchStarted(object sender, PinchStartedGestureEventArgs e)
		{
			border.Background = pinchBrush;

			initialAngle = compositeTransform.Rotation;
			initialScale = compositeTransform.ScaleX;

			messageBlock.Text = string.Format("Pinch: Angle {0} Distance {1}",
				e.Angle, e.Distance);
		}

		void HandlePinchDelta(object sender, PinchGestureEventArgs e)
		{
			compositeTransform.Rotation = initialAngle + e.TotalAngleDelta;
			compositeTransform.ScaleX 
				= compositeTransform.ScaleY = initialScale * e.DistanceRatio;

			messageBlock.Text = string.Format(
				"Pinch: DistanceRatio {0} \n TotalAngleDelta {1}",
				e.DistanceRatio, e.TotalAngleDelta);
		}

		void HandlePinchCompleted(object sender, PinchGestureEventArgs e)
		{
			border.Background = normalBrush;

			messageBlock.Text = string.Format(
				"Pinch complete \n DistanceRatio {0} \n TotalAngleDelta {1}",
				e.DistanceRatio, e.TotalAngleDelta);
		}

		void Button_Tap(object sender, System.Windows.Input.GestureEventArgs e)
		{
			ResetPosition();
		}

		const double brakeSpeed = 10;

		void HandleFlick(object sender, FlickGestureEventArgs e)
		{
			string message = string.Format("{0} Flick: Angle {1} Velocity {2}h,{3}v", 
				e.Direction, Math.Round(e.Angle), e.HorizontalVelocity, e.VerticalVelocity);
			messageBlock.Text = message;
			Debug.WriteLine(message);

			Point currentPoint = new Point(
				(double)compositeTransform.GetValue(CompositeTransform.TranslateXProperty), 
				(double)compositeTransform.GetValue(CompositeTransform.TranslateYProperty));

			double toX = currentPoint.X + e.HorizontalVelocity / brakeSpeed;
			double toY = currentPoint.Y + e.VerticalVelocity / brakeSpeed;
			Point destinationPoint = new Point(toX, toY);

			var storyboard = new Storyboard { FillBehavior = FillBehavior.HoldEnd };

			AddTranslationAnimation(
				storyboard, border, currentPoint, destinationPoint, 
				new Duration(TimeSpan.FromMilliseconds(500)),
			    new CubicEase {EasingMode = EasingMode.EaseOut});

			storyboard.Begin();
		}
		
		static void AddTranslationAnimation(Storyboard storyboard,
			FrameworkElement targetElement,
			Point fromPoint,
			Point toPoint,
			Duration duration,
			IEasingFunction easingFunction)
		{
			var xAnimation = new DoubleAnimation
									{
										From = fromPoint.X,
										To = toPoint.X,
										Duration = duration,
										EasingFunction = easingFunction
									};

			var yAnimation = new DoubleAnimation
									{
										From = fromPoint.Y,
										To = toPoint.Y,
										Duration = duration,
										EasingFunction = easingFunction
									};
				
			AddAnimation(
				storyboard,
				targetElement.RenderTransform,
				CompositeTransform.TranslateXProperty,
				xAnimation);

			AddAnimation(
				storyboard,
				targetElement.RenderTransform,
				CompositeTransform.TranslateYProperty,
				yAnimation);
		}

		static void AddAnimation(
			Storyboard storyboard,
			DependencyObject dependencyObject,
			DependencyProperty targetProperty,
			Timeline timeline)
		{
			Storyboard.SetTarget(timeline, dependencyObject);
			Storyboard.SetTargetProperty(timeline, new PropertyPath(targetProperty));
			storyboard.Children.Add(timeline);
		}

		void HandleTapAtPageLevel(object sender, GestureEventArgs e)
		{
			Debug.WriteLine("Tap handled at page level.");
		}
	}
}