﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Net;
using System.Windows;
using System.Windows.Navigation;
using System.Windows.Resources;

using Microsoft.Phone.Controls;

namespace DanielVaughan.WindowsPhone7Unleashed.Examples
{
	public partial class WebBrowserIsolatedStorageView : PhoneApplicationPage
	{
		public WebBrowserIsolatedStorageView()
		{
			InitializeComponent();
		}

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            SaveHtmlToIsolatedStorage();
            webBrowser.Base = "WebContent";
            webBrowser.Source = new Uri(@"Webpage.html", UriKind.Relative);
        }

		void SaveHtmlToIsolatedStorage()
		{
			using (IsolatedStorageFile isolatedStorageFile
				= IsolatedStorageFile.GetUserStoreForApplication())
			{
				if (!isolatedStorageFile.DirectoryExists("WebContent"))
				{
					isolatedStorageFile.CreateDirectory("WebContent");
				}

				using (IsolatedStorageFileStream isolatedStorageFileStream
					= isolatedStorageFile.OpenFile(
						@"WebContent\Webpage.html", FileMode.Create))
				{
					using (StreamWriter streamWriter 
						= new StreamWriter(isolatedStorageFileStream))
					{
						streamWriter.Write(@"<html><head></head><body>
							<h2>Stored in Isolated Storage</h2>
							<p><img src=""Images/Book.jpg"" /></p>
									</body></html>");
					}
				}

				if (!isolatedStorageFile.DirectoryExists("WebContent/Images"))
				{
					isolatedStorageFile.CreateDirectory("WebContent/Images");
				}

				StreamResourceInfo resourceInfo = Application.GetResourceStream(
					new Uri("WebBrowser/ImagePlacedInIsolatedStorage/WP7Unleashed200x.jpg", 
								UriKind.Relative));

				using (IsolatedStorageFileStream writeStream 
					= new IsolatedStorageFileStream(
							@"WebContent/Images/Book.jpg", 
							FileMode.Create, isolatedStorageFile))
				{
					CopyStreamBytes(resourceInfo.Stream, writeStream);
				}
			}
		}

		void CopyStreamBytes(
			Stream fromStream, Stream toStream, bool closeToStream = true)
		{
			if (toStream.CanWrite)
			{
				byte[] fileBytes = ReadStreamBytes(fromStream);
				toStream.Write(fileBytes, 0, fileBytes.Length);
				if (closeToStream)
				{
					toStream.Close();
				}
			}
		}

		byte[] ReadStreamBytes(Stream fileStream)
		{
			/* Read the source file into a byte array. */
			byte[] bytes = new byte[fileStream.Length];
			int readLength = (int)fileStream.Length;
			int bytesRead = 0;
			while (readLength > 0)
			{
				/* Read may return anything from 0 to readLength. */
				int read = fileStream.Read(bytes, bytesRead, readLength);

				/* When no bytes left to read it is the end of the file. */
				if (read == 0)
				{
					break;
				}

				bytesRead += read;
				readLength -= read;
			}
			return bytes;
		}

	}
}