﻿using System;
using System.ComponentModel;
using System.IO;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;

using DanielVaughan.WindowsPhone7Unleashed.ImageUpload;
using DanielVaughan.WindowsPhone7Unleashed.Windows;

using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using Microsoft.Xna.Framework.Media;

namespace DanielVaughan.WindowsPhone7Unleashed.ShareExample
{
	public partial class MainPage : PhoneApplicationPage
	{
		ImageUploadServiceClient client = new ImageUploadServiceClient();
		readonly PhotoChooserTask photoChooserTask = new PhotoChooserTask();

		public MainPage()
		{
			InitializeComponent();

			uploadCommand = new DelegateCommand(obj => UploadImage());
			
			DataContext = this;

			client.UploadImageCompleted += HandleUploadImageCompleted;
			photoChooserTask.Completed += HandlePhotoChooserTaskCompleted;
		}

		void HandlePhotoChooserTaskCompleted(object sender, PhotoResult e)
		{
			if (e.TaskResult != TaskResult.OK)
			{
				return;
			}

			imageName = e.OriginalFileName;
			Stream imageStream = e.ChosenPhoto;
			SetImageStream(imageStream);
		}

		protected override void OnNavigatedTo(NavigationEventArgs e)
		{
			base.OnNavigatedTo(e);
//			string action;
//			NavigationContext.QueryString.TryGetValue("action", out action);
//			if (action == "ShareContent")
//			{
//				/* Launched from Share menu. */
//			}
//			MessageBox.Show(action);

			/* Equivalent to: Environment.DeviceType == DeviceType.Emulator */
			if (EnvironmentValues.UsingEmulator)
			{
				ProcessShareTestImage();
			}
			else
			{
				string fieldId;
				if (NavigationContext.QueryString.TryGetValue("FileId", out fieldId))
				{
					imageName = fieldId;
					ProcessShareImage(fieldId);
				}
				else
				{
					photoChooserTask.Show();
				}
			}
		}
		
		string testImageUrl = "/DanielVaughan.WindowsPhone7Unleashed.ShareExample;component/Images/TestImage.jpg";
		string imageName = "WindowsPhone7UnleashedTestImage";

		void ProcessShareTestImage()
		{
			var imageUri = new Uri(testImageUrl, UriKind.Relative);

			using (Stream stream = Application.GetResourceStream(imageUri).Stream)
			{
				SetImageStream(stream);
			}
		}

		void SetImageStream(Stream imageStream)
		{
			BitmapSource bitmapSource = new BitmapImage();
			bitmapSource.SetSource(imageStream);
			image.Source = bitmapSource;
		}

		void ProcessShareImage(string token)
		{
			MediaLibrary mediaLibrary = new MediaLibrary();
			Picture picture = mediaLibrary.GetPictureFromToken(token);
			SetImageStream(picture.GetImage());
		}

		readonly DelegateCommand uploadCommand;

		public ICommand UploadCommand
		{
			get
			{
				return uploadCommand;
			}
		}

		void UploadImage()
		{
			BitmapImage bitmapImage = (BitmapImage)image.Source;
			if (bitmapImage == null)
			{
				return;
			}

			WriteableBitmap writeableBitmap = new WriteableBitmap(bitmapImage);
			byte[] imageBytes = writeableBitmap.ToByteArray();

			try
			{
				SetProgressBarVisibility(true);
				client.UploadImageAsync(imageBytes, imageName);
			}
			catch (Exception)
			{
				SetProgressBarVisibility(false);
				throw;
			}
		}

		void SetProgressBarVisibility(bool visible)
		{
			progressBar.Visibility = visible 
				? Visibility.Visible : Visibility.Collapsed;
		}

		void HandleUploadImageCompleted(object sender, AsyncCompletedEventArgs e)
		{
			SetProgressBarVisibility(false);
			if (e.Error != null)
			{
				MessageBox.Show("Error uploading image.");
				return;
			}
			MessageBox.Show("Image uploaded");
		}

	}
}