﻿using System;
using System.Diagnostics;
using System.Threading;

using DanielVaughan;
using DanielVaughan.InversionOfControl;
using DanielVaughan.InversionOfControl.Containers.SimpleContainer;

using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	[TestClass]
	//[Tag("NonUITests")]
	public class ChatClientTests : SilverlightTest
	{
		[AssemblyInitialize]
		public static void AssemblyInitialize()
		{
			/* Assembly initialization logic goes here. */
		}

		[AssemblyCleanup]
		public static void AssemblyCleanup()
		{
			/* Assembly cleanup logic goes here. */
		}

		[ClassInitialize]
		public static void ClassInitialize()
		{
			/* Class initialization logic goes here. */
		}

		[ClassCleanup]
		public static void ClassCleanup()
		{
			/* Class cleanup logic goes here. */
		}

		[TestInitialize]
		public void TestInitialize()
		{
			/* Test initialization logic goes here. */
		}

		[TestCleanup]
		public void TestCleanup()
		{
			/* Test cleanup logic goes here. */
		}

//		[TestInitialize]
//		public static void Initialize()
//		{
//			SimpleContainer container = new SimpleContainer();
//			container.InitializeServiceLocator();
//			Dependency.Register<IChatService>(chatService);
//		}

		[TestMethod]
		[Tag("Bug9")]
		public void ShouldSendMessage()
		{
			string testMessage = "Hello from unit test.";
			MockChatService chatService = new MockChatService();
			ChatClientViewModel viewModel = new ChatClientViewModel(chatService);
			viewModel.Message = testMessage;
			viewModel.SendCommand.Execute(null);
			Assert.AreEqual(chatService.LastMessage, testMessage);
		}

		[TestMethod]
		public void ShouldNotSendMessageIfEmpty()
		{
			MockChatService chatService = new MockChatService();
			ChatClientViewModel viewModel = new ChatClientViewModel(chatService);
			viewModel.Message = string.Empty;
			viewModel.SendCommand.Execute(null);
			Assert.IsFalse(chatService.MessageSent);
		}

		[TestMethod]
		[Priority(5)]
		public void CommandShouldBeDisabledIfMessageIfEmpty()
		{
			MockChatService chatService = new MockChatService();
			ChatClientViewModel viewModel = new ChatClientViewModel(chatService);
			viewModel.Message = string.Empty;
			Assert.IsFalse(viewModel.SendCommand.CanExecute(null));
		}
		
		[TestMethod]
		[Description(@"When the chat service receives a message, 
						the client displays it.")]
		public void CommandShouldBeEnabledIfMessageNotEmpty()
		{
			MockChatService chatService = new MockChatService();
			ChatClientViewModel viewModel = new ChatClientViewModel(chatService);
			viewModel.Message = "Test";
			Assert.IsTrue(viewModel.SendCommand.CanExecute(null));
		}

		[TestMethod]
		[TestProperty("SomeName", "SomeValue")]
		public void ShouldReceiveMessage()
		{
			string testMessage = "Hello from unit test.";
			MockChatService chatService = new MockChatService();
			ChatClientViewModel viewModel = new ChatClientViewModel(chatService);
			chatService.OnMessageReceived(new ChatMessageEventArgs(testMessage));
			CollectionAssert.Contains(viewModel.Messages, testMessage);
		}

		[TestMethod]
		[Ignore]
		[Description("This test method should be ignored.")]
		[ExpectedException(typeof(ArgumentException))]
		[Priority(1)]
		[Timeout(5000)]
		[Bug]
		public void ShouldBeSkipped()
		{
			
		}

		[Bug("TFS 123")]
		[TestMethod]
		public void ExerciseBuggyCode()
		{
			// Press F5 to skip over the failed assert.
			Assert.IsTrue(false);
		}

		[Bug("TFS 234", Fixed = true)]
		[TestMethod]
		public void ExerciseFixedCode()
		{
			Assert.IsTrue(true); /* Simulates issue resolved. */
		}

		[TestMethod]
		[Asynchronous]
		//[Timeout(100)] // uncomment to make test fail.
		[Owner("Daniel Vaughan")]
		public void ShouldFailDueToAsyncTestTimeout()
		{
			EnqueueDelay(1000);
			EnqueueTestComplete();
		}

		[TestMethod]
		[ExpectedException(typeof(ArgumentException))]
		public void ShouldFailDueToArgumentException()
		{
			// Press F5 to skip over the expected exception that occurs on the next line.
			throw new ArgumentException("Invalid argument supplied.");
		}

//		[TestMethod]
//		public void ShouldAlwaysFail()
//		{
//			Assert.Fail("Deliberate test failure.");
//			Assert.IsTrue(false);
//		}

	}
}
