﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows;

using DanielVaughan.Data;

using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DanielVaughan.WindowsPhone7Unleashed.Data
{
	[TestClass]
	public class IsolatedStorageUtilityTests : SilverlightTest
	{
		[TestMethod]
		public void CopyDatabaseToIsolatedStorage()
		{
			const string databaseName = "TestDatabase.sdf";
			IsolatedStorageUtility utility = new IsolatedStorageUtility();
			utility.CopyApplicationResourceToIsolatedStorage(databaseName, databaseName);

			Uri uri = new Uri(databaseName, UriKind.Relative);

			byte[] resourceBytes;

			using (Stream resourceStream = Application.GetResourceStream(uri).Stream)
			{
				resourceBytes = resourceStream.ReadAllBytes();
			}

			Assert.IsNotNull(resourceBytes, "Resource bytes in null");
			Assert.IsTrue(resourceBytes.Length > 0, "Resource bytes length <= 0.");

			using (IsolatedStorageFile isolatedStorageFile
							= IsolatedStorageFile.GetUserStoreForApplication())
			{
				using (IsolatedStorageFileStream outStream
					= isolatedStorageFile.OpenFile(databaseName, FileMode.Open))
				{
					byte[] isolatedStorageBytes = outStream.ReadAllBytes();
					Assert.IsNotNull(isolatedStorageBytes);
					Assert.IsTrue(isolatedStorageBytes.Length > 0);

					CollectionAssert.AreEquivalent(resourceBytes, isolatedStorageBytes);
				}
			}
		}
	}
}
