﻿using System.Collections.Generic;
using System.Data.Linq;
using System.Linq;

using DanielVaughan.WindowsPhone7Unleashed.Examples;

using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	[TestClass]
	public class BankingDataContextTests : SilverlightTest
	{
		static readonly BankingDatabaseUtility databaseUtility = new BankingDatabaseUtility();

		[ClassInitialize]
		public static void ClassInitialize()
		{
			databaseUtility.DeleteDatabase();
			databaseUtility.InitializeDatabase(true);
		}

		[ClassCleanup]
		public static void ClassCleanup()
		{
			//databaseUtility.DeleteDatabase();
		}

		[TestMethod]
		[Tag("i1")]
		public void ContextShouldReadAndWrite()
		{
			using (BankingDataContext context = databaseUtility.CreateContext())
			{
				SavingsAccount savingsAccount = new SavingsAccount {Balance = 50};
				CheckingAccount checkingAccount = new CheckingAccount
				                                  	{
				                                  		Balance = 100, 
														CheckbookCode = "12345"
				                                  	};
				context.BankAccounts.InsertAllOnSubmit(new List<BankAccount>
				                                       	{
				                                       		savingsAccount, 
															checkingAccount
				                                       	});
				context.SubmitChanges();
			}

			using (BankingDataContext context = databaseUtility.CreateContext())
			{
				List<BankAccount> accounts = context.BankAccounts.ToList();
				Assert.IsTrue(accounts.Count > 2, "There should be two accounts.");

				IQueryable<SavingsAccount> savingsAccounts 
					= context.BankAccounts.OfType<SavingsAccount>();
				Assert.IsTrue(savingsAccounts.Any());

				SavingsAccount account = savingsAccounts.First();
				Assert.IsTrue(account.Balance == 50, "Balance should be 50");
			}
		}

		[TestMethod]
		[Tag("i2")]
		[ExpectedException(typeof(ChangeConflictException))]
		public void ContextShouldEnforceUpdateChecking()
		{
			CheckingAccount beforeAccount;

			using (BankingDataContext context = databaseUtility.CreateContext())
			{
				beforeAccount = new CheckingAccount
				                	{
				                		Balance = 100, 
										CheckbookCode = "11111"
				                	};
				context.BankAccounts.InsertOnSubmit(beforeAccount);
				context.SubmitChanges();
			}

			using (BankingDataContext context = databaseUtility.CreateContext())
			{
				CheckingAccount afterAccount 
					= (CheckingAccount)context.BankAccounts.Where(
									x => x.Id == beforeAccount.Id).First();
				afterAccount.CheckbookCode = "22222";
				context.SubmitChanges();
			}

			using (BankingDataContext context = databaseUtility.CreateContext())
			{
				context.BankAccounts.Attach(beforeAccount);
				beforeAccount.CheckbookCode = "33333";
				// Press F5 to skip over the expected exception that occurs on the next line.
				context.SubmitChanges();
			}
		}
	}
}
