﻿using System;
using System.Collections.Generic;
using System.Linq;

using DanielVaughan.WindowsPhone7Unleashed.Examples;

using Microsoft.Phone.Data.Linq;
using Microsoft.Silverlight.Testing;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace DanielVaughan.WindowsPhone7Unleashed
{
	[TestClass]
	[Tag("twitter")]
	public class TwitterDataContextTests : SilverlightTest
	{
		static readonly TwitterDatabaseUtility twitterDatabaseUtility = new TwitterDatabaseUtility();

		[ClassInitialize]
		public static void ClassInitialize()
		{
			twitterDatabaseUtility.InitializeDatabase();
		}

		[ClassCleanup]
		public static void ClassCleanup()
		{
			twitterDatabaseUtility.DeleteDatabase();
		}

		[TestMethod]
		public void SaveData()
		{
			DateTime createTime = DateTime.Now;

			TwitterUser twitterUser = new TwitterUser();
			twitterUser.Id = "dbvaughan";

			using (TwitterDataContext dataContext = twitterDatabaseUtility.CreateContext())
			{
				dataContext.TwitterUsers.InsertOnSubmit(twitterUser);
				dataContext.SubmitChanges();
			}

			using (TwitterDataContext dataContext = twitterDatabaseUtility.CreateContext())
			{
				var users = from retrievedUser in dataContext.TwitterUsers
							select retrievedUser;

				Assert.IsTrue(users.Any(), "No users found.");

				var user = users.First();

				TimelineItem timelineItem01 = new TimelineItem
				                              	{
				                              		Id = "1", 
													ReceivedTime = createTime, 
													Text = "Test Tweet 01", 
													TwitterUser = user
				                              	};

				TimelineItem timelineItem02 = new TimelineItem
												{
													Id = "2",
													ReceivedTime = createTime.AddSeconds(5),
													Text = "Test Tweet 02",
													TwitterUser = user
												};

				var items = new List<TimelineItem> {timelineItem01, timelineItem02};
				dataContext.TimelineItems.InsertAllOnSubmit(items);

				dataContext.SubmitChanges();
			}

			using (TwitterDataContext dataContext = twitterDatabaseUtility.CreateContext())
			{
				var user = dataContext.TwitterUsers.Where(x => x.Id == "dbvaughan").FirstOrDefault();
				Assert.IsNotNull(user);
				//user.TimelineItems.Load();
				Assert.AreEqual(2, user.TimelineItems.Count);
				dataContext.SubmitChanges();
			}

			using (TwitterDataContext dataContext = twitterDatabaseUtility.CreateContext())
			{
				var items = from item in dataContext.TimelineItems
							select item;

				Assert.IsTrue(items.Any(), "No users found.");
				Assert.AreEqual(2, items.Count(), "Item count should be 2");
				
				var timelineItem01 = items.First();
				Assert.AreEqual("1", timelineItem01.Id);
				//Assert.IsTrue(createTime.Equals(timelineItem01.ReceivedTime));
				//Assert.AreEqual(createTime, timelineItem01.ReceivedTime);
				Assert.AreEqual("Test Tweet 01", timelineItem01.Text);
				Assert.AreEqual(twitterUser.Id, timelineItem01.TwitterUserId);

				//var timelineUser = timelineItem01.TwitterUser;
				//Assert.IsNotNull(timelineUser);
				//Assert.AreEqual("dbvaughan", timelineUser.Id);
			}
		}

		[TestMethod]
		[Tag("UpdateSchema")]
		public void UpdateSchema()
		{
			using (TwitterDataContext dataContext = twitterDatabaseUtility.CreateContext())
			{
				DatabaseSchemaUpdater updater = dataContext.CreateDatabaseSchemaUpdater();
				//				updater.AddColumn<TimelineItem>("TwitterUser2Id");
				//				updater.AddAssociation<TimelineItem>("TwitterUser2");
				//				updater.AddTable<TwitterFriend>();
				//				updater.AddIndex<TwitterFriend>("EmailAddressIndex");
				//				updater.Execute();
			}
			
		}

//		[TestMethod]
//		[Tag("UpdateSchemaVersion")]
//		public void UpdateSchemaWithVersioning()
//		{
//			using (TwitterDataContext dataContext = twitterDatabaseUtility.CreateContext())
//			{
//				DatabaseSchemaUpdater updater = dataContext.CreateDatabaseSchemaUpdater();
//				int databaseVersion = updater.DatabaseSchemaVersion;
//
//				if (databaseVersion < 2)
//				{
//					updater.AddColumn<TwitterUser>("Homepage");
//					updater.DatabaseSchemaVersion = 2;
//					updater.Execute();
//				}
//				
//				if (databaseVersion < 3)
//				{
//					updater.AddColumn<TwitterUser>("Birthday");
//					updater.DatabaseSchemaVersion = 3;
//					updater.Execute();
//				}
//			}
//		}
	}
}
