﻿using System;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Text;
using System.Threading;

namespace DanielVaughan.WindowsPhone7Unleashed.Services
{
	[AspNetCompatibilityRequirements(
		RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
	public class StockQuoteService : IStockQuoteService
	{
		Timer timer;
		bool notify = true;
		int tilePushCount;
		string subscriptionUrl;
		string stockSymbol;
		StockQuote stockQuote;
		bool sendingNotifications;
		const int notificationPeriodMs = 30000;

		public void RegisterForStockQuoteNotifications(
			PushNotificationSubscriberId pushNotificationSubscriberId, string stockSymbol)
		{
			subscriptionUrl = pushNotificationSubscriberId.PushNotificationUrl;
			this.stockSymbol = stockSymbol;
			notify = true;

			if (timer == null)
			{
				timer = new Timer(delegate
				                  	{
										/* An exception must not be raised. */
				                  		try
				                  		{
											NotifyWithStockQuote();
				                  		}
				                  		catch (Exception ex)
				                  		{
				                  			Debug.WriteLine("Unable to notify.", ex);
				                  		}
				                  	}, null, 
					2000 /* 2000 MS = 5 seconds. */, notificationPeriodMs);
			}
		}

		void NotifyWithStockQuote()
		{
			if (!notify || sendingNotifications)
			{
				return;
			}

			if (tilePushCount > 98)
			{
				tilePushCount = 0;
			}

			try
			{
				sendingNotifications = true;

				try
				{
					StockQuoter stockQuoter = new StockQuoter();
					stockQuote = stockQuoter.GetStockQuote(this.stockSymbol);
				}
				catch (Exception ex)
				{
					Debug.WriteLine("Unable to retrieve stock quote." + ex);
					return;
				}
				
				DateTime now = DateTime.Now;
				Debug.WriteLine("Sending notifications at " + now.ToLongTimeString());
				PushNotifier notifier = new PushNotifier();

				/* REMOVE. FOR SCREENSHOTS. */
				stockQuote = new StockQuote
				             	{
				             		ChangeInPrice = 3.35, 
									LastPrice = 24.05, StockSymbol = "MSFT"
				             	};

				string background;
				if (stockQuote.ChangeInPrice > 0)
				{
					background = "PushNotification/Images/StockUp.png";
				}
				else if (stockQuote.ChangeInPrice < 0)
				{
					background = "PushNotification/Images/StockDown.png";
				}
				else
				{
					background = "PushNotification/Images/StockNone.png";
				}
				
				try
				{
					notifier.SendTileNotification(
						stockQuote.ToString(), 
						++tilePushCount, 
						background, 
						subscriptionUrl, 
						PushNotificationPriority.RealTime);
				}
				catch (Exception ex)
				{
					Debug.WriteLine("Unable to send tile notifications:" + ex);
				}

				try
				{
					notifier.SendToastNotification("Stock Quote", stockQuote.ToString(), 
						subscriptionUrl, PushNotificationPriority.RealTime);
				}
				catch (Exception ex)
				{
					Debug.WriteLine("Unable to send toast notification:" + ex);
				}

				try
				{
					notifier.SendRawNotification(stockQuote, 
						subscriptionUrl, PushNotificationPriority.RealTime);
				}
				catch (Exception ex)
				{
					Debug.WriteLine("Unable to send raw notification: " + ex);
				}
			}
			finally
			{
				sendingNotifications = false;
			}
		}

		public void UnregisterForStockQuoteNotifications(
			PushNotificationSubscriberId pushNotificationSubscriberId)
		{
			notify = false;
			tilePushCount = 0;
		}

	    public string TestServer(string foo)
	    {
	        Debug.WriteLine("TestServer method." + foo);
	        return foo;
	    }
	}
}
