using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;

namespace DanielVaughan.WindowsPhone7Unleashed.Services
{
	class StockQuoter
	{
		public StockQuote GetStockQuote(string stockSymbol)
		{
			string queryString = string.Format("s={0}&f=sl1p2d1t1c1hgvba", stockSymbol);
			string url = "http://download.finance.yahoo.com/d/quotes.csv?" + queryString;
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);

			string responseString;
			using (HttpWebResponse response = (HttpWebResponse)request.GetResponse())
			{
				using (var reader = new StreamReader(response.GetResponseStream(), Encoding.ASCII))
				{
					responseString = reader.ReadLine();
				}
			}

			Debug.WriteLine("Stock Quote: " + responseString);

			var cells = responseString.Split(',');
			if (cells.Length < 3)
			{
				throw new Exception("Invalid response.");
			}
			if (cells[1].ToLower() == @"\na\")
			{
				throw new UnknownStockSymbolException(stockSymbol);
			}

			StockQuote stockQuote = new StockQuote();
			string changeInPrice = cells[2].Replace("\"", "").Replace("%", "");
			stockQuote.ChangeInPrice = double.Parse(changeInPrice, 
				NumberStyles.AllowLeadingSign | NumberStyles.AllowDecimalPoint);
			stockQuote.LastPrice = double.Parse(cells[1].Replace("\"", ""));
			stockQuote.StockSymbol = cells[0].Replace("\"", "");
			return stockQuote;
		}
	}

	public class UnknownStockSymbolException : Exception
	{
		public UnknownStockSymbolException(string message) : base(message)
		{
			/* Intentionally left blank. */
		}
	}

}